(* :Package Version: 1.10.20 *)

(* :Copyright: Copyright 2002, Selwyn Hollis *)

(* :Name: Graphics`DEGraphics` *)
 
(* :Author: Selwyn Hollis *)

(*:Summary:
This package defines the functions NDPlot, PhasePlot, SlopeField, TimeStatePlot,
PoincareTimeSection, ViewProjections, and PlotImplicit.
*)

BeginPackage["DiffEqs`DEGraphics`"]

NDPlot::usage = "NDPlot[DEs, fns, {t,tmin,tmax}, options].
Uses NDSolve to compute a numerical solution of a system of first-order differential equations
and, if there are 1, 2, or 3 equations, plots the solution. 
Returns {solution, -Graphics-}. Argument syntax is the same as for NDSolve.
Uses the same options as NDSolve, Plot, ParametricPlot, and ParametricPlot3D.";

PhasePlot::usage = "PhasePlot[{f,g}, {t,tmin,tmax}, {x,xmin,xmax}, {y,ymin,ymax}, options].
By default, four solution curves are drawn. The InitialPoints option can be
used to specify a list of starting points for curves drawn by PhasePlot. Each initial
point may give rise to multiple curves as specified by the MaxCurves option.";

DEPlot::usage = "DEPlot[f, {t,tmin,tmax}, {y,ymin,ymax}, options].
By default, four solution curves are drawn. The InitialPoints option can be
used to specify a list of starting points for curves drawn by DEPlot. Each initial
point may give rise to multiple curves as specified by the MaxCurves option.";

TimeStatePlot::usage = "TimeStatePlot[{x[t],y[t]}, {t,tmin,tmax}, options]. Creates a 3D plot
of the time-state trajectory (t,x,y) with projections onto the tx, ty, and xy planes.";

PoincareTimeSection::usage = "PoincareTimeSection[{f,g}, {t,tmin,tmax,dt}, {x,x0}, {y,y0}, options].
Creates a Poincare time section plot for the system x'=f, x(t0)=x0, y'=g, y(t0)=y0, with time step dt.";

ViewProjections::usage = "ViewProjections[{f[t],g[t],h[t]}, {t,tmin,tmax}, {x,y,z}, options].
Creates a GraphicsArray of 2-D projections of the curve {f[t],g[t],h[t]} onto the xy, xz, and yz coordinate planes.";

PlotImplicit::usage = "PlotImplicit[F[t,y], {t,tmin,tmax}, {y,ymin,ymax}, options].
Creates a contour plot of the function F."

Arrows::usage = "An option of PhasePlot and DEPlot. Specifies the number of arrows to draw in each
direction. The default is {11,9}.";

ScaleArrows::usage = "An option of PhasePlot and DEPlot. A scale factor for the length of the
arrows. The default is 0.25.";

ArrowColor::usage = "An option of PhasePlot and DEPlot. Specifies the color of the arrows. The
default is RGBColor[0,.6,.6].";

InitialPoints::usage = "An option of PhasePlot and DEPlot. A list of one or more initial points through
which solution curves are drawn. (See MaxCurves.)" 

Gap::usage = "An option of PhasePlot and DEPlot. Controls the spacing between initial points on 
automatically generated curves. The default is 0.1. The actual spacing is Gap*diag, where
diag=Sqrt[(xmax-xmin)^2+(ymax-ymin)^2]";

MaxCurves::usage = "An option of PhasePlot and DEPlot. Specifies the number of automatically drawn
curves arising from each seed point. The default is {1}.";

StepSize::usage = "An option of PhasePlot and DEPlot. Specifies the stepsize used in the numerical
method or, in case Method->Automatic, the length of the intervals over which NDSolve is used to 
compute the solution. The default is 0.1.";

ShowStartPoints::usage = "An option of PhasePlot and DEPlot. If set to True, then the starting
point for each curve is plotted as a red dot. The default is False.";

ShowEquilibriumPoints::usage = "An option of PhasePlot and DEPlot. If set to True, then equilibrium
points are plotted as blue dots. The default is False. Equilibrium points are computed
with Solve, which will fail in many cases";

PrintEquilibriumPoints::usage = "An option of PhasePlot. The default is False.";

ShowNumbers::usage = "An option of PhasePlot. The default is False. Use sparingly.";

UseVectorField::usage = "An option of PhasePlot and DEPlot. If set to True, then the vector field
is drawn rather than the direction field. The default is False."

ShowNullclines::usage = "An option of PhasePlot. The default is False.";

Isoclines::usage = "An option of DEPlot. The default is False. If set to True, then 10 isoclines
are drawn. If Isoclines->{m1,m2,...,mn}, then isoclines f[t,y]=mi, i=1,...,n, are drawn.";

ShadowColors::usage = "An option of TimeStatePlot. Specifies the colors of the projections onto 
the tx, ty, and xy planes. The default is {RGBColor[1,0,0], RGBColor[0,1,0], RGBColor[0,0,1]}.";

ShadowPositions::usage = "An option of TimeStatePlot. Specifies the positions of the planes containing 
the planar projections. Contains values passed on to Shadow (in Graphics`Graphics3D`) as XShadowPosition,
YShadowPosition, ZShadowPosition. The default is {-.67,.8,-.67}.";

OuterFrame::usage = "An option of ViewProjections. The default is False.";
OuterFrameStyle::usage = "An option of ViewProjections. The default is None.";

Options[PhasePlot] := {ArrowColor-> RGBColor[0, 0.6, 0.6], Arrows-> {11, 9}, AspectRatio-> 0.75,
	Axes-> Automatic, Axes-> True, AxesLabel-> None, AxesOrigin-> Automatic, AxesStyle-> Automatic,
	Background-> Automatic, ColorOutput-> Automatic, DefaultColor-> Automatic, Epilog-> {}, Frame-> False, 
	FrameLabel-> None, FrameStyle-> Automatic, FrameTicks-> Automatic, Gap-> 0.1, GridLines-> None, 
	HaltingRadius->Automatic, ImageSize-> Automatic, InitialPoints-> Automatic, MaxCurves-> {1},
	Method-> RungeKutta, PlotJoined-> True, PlotLabel-> None, PlotRange-> All, PlotRegion-> Automatic, 
	PlotStyle-> Thickness[0.006], Prolog-> {}, RotateLabel-> True, ScaleArrows-> 1, 
	ShowEquilibriumPoints-> False, ShowField-> True, ShowNullclines->False, ShowNumbers->False, 
	ShowStartPoints-> False, ShowLastPoints-> False, StepSize-> 0.1, Ticks-> Automatic,
	UseVectorField-> False, DefaultFont :> $DefaultFont, DisplayFunction :> $DisplayFunction,
	FormatType :> $FormatType, TextStyle :> $TextStyle};
  
Options[DEPlot] := {ArrowColor-> RGBColor[0, 0.6, 0.6], Arrows-> {11, 9}, AspectRatio-> 0.75,
	Axes-> Automatic, Axes-> True, AxesLabel-> None, AxesOrigin-> Automatic, AxesStyle-> Automatic,
	Background-> Automatic, ColorOutput-> Automatic, DefaultColor-> Automatic, Epilog-> {}, Frame-> False, 
	FrameLabel-> None, FrameStyle-> Automatic, FrameTicks-> Automatic, Gap-> 0.1, GridLines-> None, 
	ImageSize-> Automatic, InitialPoints-> Automatic, Isoclines-> False, MaxCurves-> {1}, Method-> RungeKutta, 
	PlotJoined-> True, PlotLabel-> None, PlotRange-> All, PlotRegion-> Automatic, 
	PlotStyle-> Thickness[0.006], Prolog-> {}, RotateLabel-> True, ScaleArrows-> 1, 
	ShowEquilibriumPoints-> False, ShowField-> True, ShowNumbers->False, ShowStartPoints-> False,
	StepSize-> 0.1, Ticks-> Automatic, UseVectorField-> False, DefaultFont :> $DefaultFont,
	DisplayFunction :> $DisplayFunction, FormatType :> $FormatType, TextStyle :> $TextStyle};
  
Options[TimeStatePlot] :={AspectRatio-> Automatic, Axes-> True, AxesEdge-> Automatic, AxesLabel-> None,
	AxesStyle-> Automatic, Background-> Automatic, Boxed-> True, BoxRatios-> {2, 1, 1},
	BoxStyle-> Automatic, ColorOutput-> Automatic, Compiled-> True, DefaultColor-> Automatic, Epilog-> {},
	FaceGrids-> None, ImageSize-> Automatic, Lighting-> True, PlotLabel-> None, PlotPoints-> Automatic,
	PlotRange-> All, PlotRegion-> {{0.025, 0.975}, {0.025, 0.975}}, PlotStyle-> Automatic, Prolog-> {},
	RenderAll-> True, ShadowColors-> {RGBColor[1, 0, 0], RGBColor[0, 1, 0], RGBColor[0, 0, 1]},
	ShadowPositions-> {-0.67, 0.8, -0.67}, Ticks-> Automatic, ViewCenter-> Automatic, ViewPoint-> {2, -3, 1},
	DefaultFont :> $DefaultFont, DisplayFunction :> $DisplayFunction, FormatType :> $FormatType,
	TextStyle :> $TextStyle};
 
Options[PlotImplicit] := 
	{AspectRatio-> 1/GoldenRatio, Axes-> False, AxesLabel-> None, AxesOrigin-> Automatic,
	AxesStyle-> Automatic, Background-> Automatic, ColorFunction->(Hue[ArcTan[#]]&), ColorFunctionScaling-> True,
	ColorOutput-> Automatic, Compiled-> True, ContourLines-> True, Contours-> 10, ContourShading-> False,
	ContourSmoothing-> True,  ContourStyle-> Automatic, DefaultColor-> Automatic, Epilog-> {},
	Frame-> True, FrameLabel-> None, FrameStyle-> Automatic, FrameTicks-> Automatic, ImageSize-> Automatic,
	Mesh-> False, MeshStyle-> Automatic, PlotLabel-> None, PlotPoints-> 15, PlotRange-> Automatic, 
	PlotRegion-> Automatic, Prolog-> {}, RotateLabel-> True, ShowDensity-> False, Ticks-> Automatic,
	DefaultFont:> $DefaultFont, DisplayFunction:> $DisplayFunction, FormatType:> $FormatType,
	TextStyle:> $TextStyle};

Options[ViewProjections] := {AspectRatio-> 1/GoldenRatio, Axes-> Automatic, 
	AxesOrigin-> Automatic, AxesStyle-> Automatic, Background-> Automatic, ColorOutput-> Automatic,
	Compiled-> True, DefaultColor-> Automatic, Epilog-> {}, Frame-> False, FrameLabel-> None,
	FrameStyle-> Automatic, FrameTicks-> Automatic, GraphicsSpacing-> 0.1, GridLines-> None,
	ImageSize->{72*6, Automatic}, MaxBend-> 10., OuterFrame-> False, OuterFrameStyle-> {}, PlotDivision-> 30.,
	PlotLabel-> None, PlotPoints-> 25, PlotRange-> Automatic, PlotRegion-> Automatic, PlotStyle-> Automatic,
	Prolog-> {}, Ticks-> Automatic, DefaultFont:> $DefaultFont, DisplayFunction:> $DisplayFunction,
	FormatType:> $FormatType, TextStyle:> $TextStyle};

Options[NDPlot]:= {AccuracyGoal-> Automatic, AspectRatio-> 1/GoldenRatio, Axes-> Automatic, AxesEdge-> Automatic,
	AxesLabel-> None, AxesOrigin-> Automatic, AxesStyle-> Automatic, Background-> Automatic, Boxed-> True,
	BoxRatios-> Automatic, BoxStyle-> Automatic, ColorOutput-> Automatic, Compiled-> True,
	DefaultColor-> Automatic, Epilog-> {}, FaceGrids-> None, Frame-> False, FrameLabel-> None,
	FrameStyle-> Automatic, FrameTicks-> Automatic, GridLines-> None, ImageSize-> Automatic,
	InterpolationPrecision-> Automatic, MaxBend-> 10., MaxRelativeStepSize-> 1/10, MaxSteps-> Automatic,
	MaxStepSize-> Infinity, Method-> Automatic, PlotDivision-> 30., PlotLabel-> None, PlotPoints-> 30,
	PlotRange-> Automatic, PlotRegion-> Automatic, PlotStyle-> Automatic, PrecisionGoal-> Automatic,
	Prolog-> {}, RotateLabel-> True, StartingStepSize-> Automatic, StoppingTest-> None, Ticks-> Automatic,
	ViewCenter-> Automatic, ViewPoint-> {1.3,-2.4, 2.}, ViewVertical-> {0.,0.,1.}, WorkingPrecision-> 16,
	DefaultFont:> $DefaultFont, DisplayFunction:> $DisplayFunction, FormatType:> $FormatType, TextStyle:> $TextStyle};
	
Options[PoincareTimeSection]:= Union[Options[ListPlot],Options[NDSolve]];
 
IsoclinePoints::usage = " ";
IsoclineStyle::usage = " ";
HaltingRadius::usage = " ";

Begin["`Private`"]

<<Graphics`PlotField`;
<<Graphics`Graphics3D`;
<<Utilities`FilterOptions`

Off[Graphics::"gptn"];


(*** Definition of NDPlot ***)

NDPlot[de_List, y_[t_Symbol], {t_Symbol, t1_?NumericQ, t2_?NumericQ}, opts___Rule]:=
Module[{soln},
	soln=y[t]/.Flatten[NDSolve[de,Flatten[y[t]],{t,t1,t2},
   		FilterOptions[NDSolve,opts], FilterOptions[NDSolve, Options[NDPlot]]]];
    soln=If[Head[soln]=!=List,{soln},soln];
    {soln,
    	Which[
    	Length[soln]==1,
			Plot[Evaluate[soln],{t,t1,t2},
       			Evaluate[FilterOptions[Plot,opts], FilterOptions[Plot, Options[NDPlot]]]],
		Length[soln]==2,
       		 ParametricPlot[Evaluate[soln],{t,t1,t2},
        		  Evaluate[FilterOptions[ParametricPlot,opts], FilterOptions[ParametricPlot, Options[NDPlot]]]],
        Length[soln]==3,
        ParametricPlot3D[Evaluate[Append[soln, PlotStyle/.{opts}/.{PlotStyle->{}}]],{t,t1,t2},
          Evaluate[FilterOptions[ParametricPlot3D,opts], AspectRatio->Automatic,
          FilterOptions[ParametricPlot3D, Options[NDPlot]]]]
        	]
		}
	];
          

(*** Definition of PhasePlot ***)

PhasePlot[{ff_, gg_}, {t_Symbol, t1_?NumericQ, t2_?NumericQ},
		  {x_Symbol, x1_?NumericQ, x2_?NumericQ}, {y_Symbol, y1_?NumericQ, y2_?NumericQ},
			opts___Rule] :=
Module[{arrows, autonomous, backwardplots, backwardpoints, 
	drawfield, endpoints, eqpoints, eqpts={}, f, field, forwardplots, forwardpoints, 
	fuzz, g, gap, h, hh, i, isoclines, isopoints, isos, isostyle, lastpoints, k, m, 
	maxcurves, method, n, ndsopts, nonautonomouspair, numcurves, perp, plotrange, 
	plotregion, printeqpts, scalearrows, scalefunction, seeds, showeqpts, plotstyle,
	showfield, showisoclines, showlast, shownullclines, slopes, start, startpoints, step, 
	stepsize, tmax, tmin, u, usevectorfield, v, w, xmax, xmaxminus, xmaxplus, xmin, 
	xminminus, xminplus, xnulls, ymax, ymaxminus, ymaxplus, ymin, yminminus, yminplus, 
	ynulls, a, b, c, d, fc, gc, uc, DPstep, allpoints, returnpoints},


PhasePlot::"tint"="Time interval must include zero.";
If[t1<=0<=t2, {},
Message[PhasePlot::"tint"];Return[$Failed]];

PhasePlot::"num"="Vector field does not provide numerical values.";
If[And@@NumericQ/@({ff,gg}/.{t->0,x->N[x1],y->N[y1]}),{}, Message[PhasePlot::"num"];Return[$Failed]];

f=ff/.{x[t]->x,y[t]->y}; g=gg/.{x[t]->x,y[t]->y};

fc=Compile[{tt,xx,yy},Evaluate[f//.{t-> tt, x-> xx, y-> yy}]];
gc=Compile[{tt,xx,yy},Evaluate[g//.{t-> tt, x-> xx, y-> yy}]];

showfield = ShowField /. {opts} /. Options[PhasePlot];
usevectorfield = UseVectorField /. {opts} /. Options[PhasePlot];
usevectorfield = If[usevectorfield=!=True && usevectorfield=!=False, False, usevectorfield];
arrows = Arrows /. {opts} /. Options[PhasePlot];
arrowcolor = ArrowColor /. {opts} /. Options[PhasePlot];
scalearrows = .25 ScaleArrows /. {opts} /. Options[PhasePlot];
scalefunction = If[usevectorfield, None, (1.&)];

shownumbers= ShowNumbers/. {opts}/.  Options[PhasePlot];


showstartpoints = ShowStartPoints /. {opts} /. Options[PhasePlot];
showeqpts = ShowEquilibriumPoints /. {opts} /. Options[PhasePlot];
showlast = ShowLastPoints /. {opts} /. Options[PhasePlot];
showlast = If[showlast=!=True, False, True];
shownullclines = ShowNullclines /. {opts} /. Options[PhasePlot];

isoclines = Isoclines /. {opts};
isoclines = If[isoclines===Isoclines, False, isoclines];
showisoclines = If[isoclines===False || isoclines===None, False, True];
showisoclines = If[x=!=t, False, showisoclines];
slopes = If[isoclines===True || isoclines===Automatic, 10, isoclines];

(* secret options *) 

	isostyle = IsoclineStyle /. {opts} /. {IsoclineStyle->GrayLevel[.3]};
	isopoints = IsoclinePoints /. {opts} /. {IsoclinePoints->45};
	printeqpts = PrintEquilibriumPoints /. {opts} /. {PrintEquilibriumPoints->False};
	If[printeqpts=!=True, False, True];
	fuzz = HaltingRadius /. {opts} /. Options[PhasePlot];
	fuzz= If[Not[NumericQ[fuzz]],10^(-6)((x1-x2)^2 + (y1-y2)^2), fuzz^2];

maxcurves = MaxCurves /. {opts} /. Options[PhasePlot];
maxcurves= If[Head[maxcurves]===Integer, {maxcurves}, maxcurves];
gap = Gap /. {opts} /. Options[PhasePlot];

PhasePlot::"badseeds"="InitialPoints must be Automatic, None, or a list of one or more points. Reverting to default.";

seeds = InitialPoints /. {opts} /. Options[PhasePlot];
seeds=If[maxcurves===None || maxcurves===0 || maxcurves==={0}, None, seeds];
seeds=If[seeds===Automatic,
N[{{(3x1+x2)/4,(2y1+y2)/3},{(3x1+x2)/4,(y1+2y2)/3},{(x1+3x2)/4,(2y1+y2)/3},{(x1+3x2)/4,(y1+2y2)/3}}],
seeds];
seeds=If[seeds=!=None && Not[MatrixQ[seeds]],
Message[PhasePlot::"badseeds"];
N[{{(3x1+x2)/4,(2y1+y2)/3},{(3x1+x2)/4,(y1+2y2)/3},{(x1+3x2)/4,(2y1+y2)/3},{(x1+3x2)/4,(y1+2y2)/3}}],
seeds];

stepsize = Stepsize /. {opts};
stepsize=If[NumberQ[N[stepsize]], stepsize, StepSize/.{opts}/.Options[PhasePlot]];
method = Method /. {opts} /. Options[PhasePlot];

PhasePlot::"method"="Method must be Euler, Heun, RungeKutta, DormandPrince, or Automatic.";
If[MemberQ[{"Euler","Heun","RungeKutta","DormandPrince","Automatic"}, method//ToString], {},
Message[DEGraphics::"method"];Return[$Failed]];

(* Process PlotRange. *)

{tmin, tmax, xmin, xmax, ymin, ymax} = N[{t1, t2, x1, x2, y1, y2}];

plotrange= PlotRange/.{opts}/.{PlotRange->N[{{x1, x2}, {y1, y2}}]};

{{xmin, xmax}, {ymin, ymax}}= N[
	Which[	MatrixQ[plotrange],
				plotrange,
			Head[plotrange]===List && plotrange[[2]]===Automatic,
				{plotrange[[1]],{y1,y2}},
			Head[plotrange]===List && plotrange[[1]]===Automatic,
				{{x1,x2},plotrange[[2]]},
			VectorQ[plotrange],
				{{x1,x2},plotrange},
			1==1,
				{{x1, x2}, {y1, y2}}
		]];

{{xminplus, xmaxminus},{yminplus, ymaxminus}}= {{xmin,xmax},{ymin,ymax}}.{{0.975,0.025},{0.025,0.975}};
{{xminminus,xmaxplus},{yminminus,ymaxplus}}= {{xmin,xmax},{ymin,ymax}}.{{1.025,-0.025},{-0.025,1.025}};

plotrange= PlotRange/.{opts}/.{PlotRange->{{xminminus,xmaxplus},{yminminus,ymaxplus}}};
plotrange= 
	Which[	Head[plotrange]===List && plotrange[[2]]===Automatic,
				{plotrange[[1]],{yminminus,ymaxplus}},
			Head[plotrange]===List && plotrange[[1]]===Automatic,
				{{xminminus,xmaxplus},plotrange[[2]]},
			VectorQ[plotrange],
				{{xminminus,xmaxplus},plotrange},
			plotrange===Automatic,
				{{xminminus,xmaxplus},{yminminus,ymaxplus}},
			1==1,
				plotrange
		];
		
{a,b,c,d}={Max[x1,xmin],Min[x2,xmax],Max[y1,ymin],Min[y2,ymax]};

(* Finished processing options. Now make the field plot. *)

nonautonomouspair = If[t===x, False, (D[f,t]=!= 0 || D[g,t]=!= 0)];
drawfield=If[nonautonomouspair||arrows===0||arrows==={0,0}||arrows==={0}, False, True];
showeqpts=If[nonautonomouspair, False, showeqpts];
shownullclines=If[nonautonomouspair, False, shownullclines];

showfield=If[showfield=!=True, False, True];

field= If[drawfield && showfield,
	PlotVectorField[{f, g}, {x, xminplus, xmaxminus}, {y, yminplus, ymaxminus},
		PlotPoints-> arrows, ScaleFactor-> scalearrows,
		ScaleFunction-> scalefunction, ColorFunction-> (arrowcolor &),
		DisplayFunction-> Identity, ArrowCenter->.67],
	{}];

(* Find equilibria	*)
	
If[t=!=x,
	eqpts= {x,y}/.Solve[{f==0,g==0},{x,y}];
	eqpts= If[eqpts==={x,y}, {},
					Select[eqpts,(Element[N[#[[1]]],Reals] && Element[N[#[[2]]],Reals])&]];
					If[printeqpts && eqpts=!={}, Print["Equilibrium points are ", eqpts]],
	If[showeqpts,
		eqpts= {0,y}/.Solve[{g==0},y];
		eqpts= If[eqpts==={0,y}, {}, Select[eqpts, Element[N[#[[2]]], Reals]&]];
					If[printeqpts && eqpts=!={0,y}, Print["Equilibria are ", eqpts]],
		eqpts={}]
	];
	
eqpts= If[eqpts==={x,y} || eqpts==={0,y} || eqpts ==={},  {},
				DeleteCases[eqpts, _?((#[[1]]<xmin || #[[1]]>xmax || #[[2]]<ymin || #[[2]]>ymax)&)]];

eqpoints=If[showeqpts && eqpts=!={},
				Graphics[Join[{PointSize[0.02], RGBColor[0,0,1]}, Point/@eqpts]],
		{}];
	

(*
	Beginning of big If:
*)

If[seeds===None, forwardplots={}; backwardplots={}; startpoints={},

(*
   Otherwise generate starting points based on seeds, maxcurves, and gap.
*)

SetAttributes[LessEqual,Listable];

w = gap*Sqrt[(xmax - xmin)^2 + (ymax - ymin)^2];

numcurves =
If[Length[maxcurves] == 1, Table[maxcurves[[1]], {Length[seeds]}], maxcurves];

perp = {-g,f}/Sqrt[f^2 + g^2 + 10^(-6)] /. t-> 0;

start = {};
Do[
	start = Join[{seeds[[i]]}, start, {seeds[[i]]}];
	m = Length[start];

	While[
		Length[start] - m + 1 <= numcurves[[i]]/2 &&
			First[Union[{a,c} <= Last[start] <= {b,d}]],
		v = perp /. {x-> Last[start][[1]], y-> Last[start][[2]]};
		start = Append[start, Last[start] + w*v]];
	
	m = Length[start];

	While[
		Length[start] - m + 1 <= numcurves[[i]]/2 &&
			First[Union[{a,c} <= First[start] <= {b,d}]],
		v = perp /. {x-> start[[1,1]], y-> start[[1,2]]};
		start = Prepend[start, start[[1]] - w*v]],
	{i, 1, Length[seeds]}];
	
start= DeleteCases[start, _?((#[[1]]<a || #[[1]]>b || #[[2]]<c || #[[2]]>d)&)];

start = Union[start, If[t===x, eqpts,{}]];

(*
	Define a step of the numerical method to be used.
*)

If[(method//ToString)==="DormandPrince",
	u[tt_, {xx_, yy_}] := {fc[tt,xx,yy], gc[tt,xx,yy]};
	DPstep = Compile[{t0, x0, y0, hh},
	 Module[{dpA = {{0,0,0,0,0,0,0,0,0,0,0,0,0},{1/18,0,0,0,0,0,0,0,0,0,0,0,0},
		{1/48,1/16,0,0,0,0,0,0,0,0,0,0,0},{1/32,0,3/32,0,0,0,0,0,0,0,0,0,0},
		{5/16,0,-(75/64),75/64,0,0,0,0,0,0,0,0,0},{3/80,0,0,3/16,3/20,0,0,0,0,0,0,0,0},
		{29443841/614563906,0,0,77736538/692538347,-28693883/1125000000,23124283/1800000000,0,0,0,0,0,0,0},
		{16016141/946692911,0,0,61564180/158732637,22789713/633445777,545815736/2771057229,-180193667/1043307555,0,0,0,0,0,0},
		{39632708/573591083,0,0,-433636366/683701615,-421739975/2616292301,100302831/723423059,790204164/839813087,
		800635310/3783071287,0,0,0,0,0},{246121993/1340847787,0,0,-37695042795/15268766246,-309121744/1061227803,
		-12992083/490766935,6005943493/2108947869,393006217/1396673457,123872331/1001029789,0,0,0,0},
		{-1028468189/846180014,0,0,8478235783/508512852,1311729495/1432422823,-10304129995/1701304382,-48777925059/3047939560,
		15336726248/1032824649,-45442868181/3398467696,3065993473/597172653,0,0,0},{185892177/718116043,0,0,
		-3185094517/667107341,-477755414/1098053517,-703635378/230739211,5731566787/1027545527,5232866602/850066563,
		-4093664535/808688257,3962137247/1805957418,65686358/487910083,0,0},{403863854/491063109,0,0,-5068492393/434740067,
		-411421997/543043805,652783627/914296604,11173962825/925320556,-13158990841/6184727034,3936647629/1978049680,
		-160528059/685178525,248638103/1413531060,0,0}},
       dpB = {14005451/335480064,0,0,0,0,-59238493/1068277825,181606767/758867731,561292985/797845732,
        	  -1041891430/1371343529,760417239/1151165299,118820643/751138087,-528747749/2220607170,1/4},
       dpC = {0,1/18,1/12,1/8,5/16,3/8,59/400,93/200,5490023248/9719169821,13/20,1201146811/1299019798,1,1},
        k},
        k = Table[{0.,0.}, {13}]; 
        k[[1]] = hh*u[t0, {x0,y0}];
    	Do[k[[i]] = hh*u[t0 + dpC[[i]]*hh, {x0,y0} + dpA[[i]].k], {i, 2, 13}]; 
     	{x0,y0} + dpB.k]
     	];
   step[t0_,x0_,y0_,hh_] := Flatten[{t0+hh, DPstep[t0,x0,y0,hh]}],
   (* else *)
   u[{tt_, xx_, yy_}] := {1, fc[tt,xx,yy], gc[tt,xx,yy]}
   ];

If[(method//ToString)==="Automatic",
	ndsopts=FilterOptions[NDSolve,opts];
	step[t0_,x0_,y0_,hh_] :=Module[{soln},
		soln={t0 + hh, {xx[tt], yy[tt]} /. 
    		NDSolve[{xx'[tt] == f /. {t->tt, x-> xx[tt], y-> yy[tt]}, 
					 yy'[tt] == g /. {t->tt, x-> xx[tt], y-> yy[tt]}, 
					 xx[t0] == x0,
					 yy[t0] == y0},
					 {xx[tt], yy[tt]}, {tt, t0, t0 + hh},
					 MaxStepSize -> Abs[hh]/2, ndsopts]};
		Flatten[soln /. tt-> t0 + hh]],
step = 
	Which[
	(method//ToString)==="Euler",
		Compile[{t0,x0,y0,hh}, {t0,x0,y0} + hh*u[{t0,x0,y0}]],
	(method//ToString)==="Heun",
		Compile[{t0,x0,y0,hh}, Module[{k1, k2},
			k1 = u[{t0,x0,y0}];
			k2 = u[{t0,x0,y0} + hh*k1];
			{t0,x0,y0} + hh*(k1 + k2)/2]],
	(method//ToString)==="RungeKutta",
		Compile[{t0,x0,y0,hh}, Module[{k1, k2, k3, k4},
			k1 = u[{t0,x0,y0}];
			k2 = u[{t0,x0,y0} + hh*k1/2];
			k3 = u[{t0,x0,y0} + hh*k2/2];
			k4 = u[{t0,x0,y0} + hh*k3];
			{t0,x0,y0} + hh*(k1 + 2*k2 + 2*k3 + k4)/6
		]],
		(method//ToString)==="DormandPrince",
		step]
];


(*
	Compute points forward in time and build plots.
*)

n = Length[start];
h = N[stepsize];

forwardpoints = If[eqpts==={} || t===x,
	Table[NestWhileList[Apply[step,Flatten[{#,h}]]&,
		Join[{If[t===x,start[[k]][[1]],0]},start[[k]]],
		First[Union[{tmin,a,c} <= # <= {tmax-h/3,b,d}]]&],
	{k,n}],
	Table[NestWhileList[Apply[step,Flatten[{#,h}]]&,
		Join[{If[t===x,start[[k]][[1]],0]},start[[k]]],
		(First[Union[{tmin,a,c} <= # <= {tmax-h/3,b,d}]]&&
		minsquaredistance[#[[{2,3}]],eqpts] > fuzz)&  (*minsquaredistance is defined below*)
			],
	{k,n}]];

allpoints= If[shownumbers, forwardpoints,{}];
forwardpoints = Map[Rest, forwardpoints, {2}];

lastpoints = DeleteCases[Last/@forwardpoints, _?((#[[1]]<a || #[[1]]>b || #[[2]]<c || #[[2]]>d)&)];

forwardpoints = clip[#,{{a,b},{c,d}}]&/@forwardpoints; (*clip is defined below*)

forwardplots =
Table[
	ListPlot[forwardpoints[[k]],
		DisplayFunction-> Identity,
		FilterOptions[ListPlot,opts],
		FilterOptions[ListPlot, Options[PhasePlot]]
		],
{k, n}];

(*
	Compute points backward in time and build plots.
*)

h = -N[stepsize];

backwardpoints =
Table[NestWhileList[Apply[step,Flatten[{#,h}]]&,
	Join[{If[t===x,start[[k]][[1]],0]},start[[k]]],
	First[Union[{t1-h/3,a,c} <= # <= {tmax,b,d}]]&],{k,n}];

If[shownumbers, allpoints = Join[backwardpoints,allpoints];
allpoints = If[t===x, allpoints//.{{p1_,p2_,p3_}->{p2,p3}}, allpoints]];

backwardpoints = Map[Rest, backwardpoints, {2}];
backwardpoints = clip[#,{{a,b},{c,d}}]&/@backwardpoints;  (* clip is defined below*)

backwardplots =
Table[
	ListPlot[backwardpoints[[k]],
		DisplayFunction-> Identity,
		FilterOptions[ListPlot,opts],
		FilterOptions[ListPlot, Options[PhasePlot]]
		],
{k, n}];

ClearAttributes[LessEqual,Listable];	

]; (* end of big If *)

	
(*
	Create extra stuff for the plot as requested.
*)
	
endpoints=If[showlast,Graphics[Join[{PointSize[0.02], RGBColor[.6,0,.6]}, Point /@ lastpoints]], {}];

startpoints = If[showstartpoints, Graphics[Join[{PointSize[0.013], Hue[1]}, Point /@ start]], {}];	

xnulls:= If[shownullclines,
		ContourPlot[Evaluate[g],{x,xmin,xmax},{y,ymin,ymax},
			Frame->False, Contours->{0}, ContourShading->False,
			ContourStyle->isostyle, PlotPoints->isopoints, DisplayFunction->Identity],
		{}];
		
ynulls:= If[shownullclines, 
		ContourPlot[Evaluate[f],{x,xmin,xmax},{y,ymin,ymax},
			Frame->False, Contours->{0}, ContourShading->False,
			ContourStyle->isostyle, PlotPoints->isopoints, DisplayFunction->Identity],
		{}];

isos:= If[showisoclines, 
		ContourPlot[Evaluate[g],{x,xmin,xmax},{y,ymin,ymax},
			Frame->False, Contours->slopes, ContourShading->False,
			ContourStyle->isostyle, PlotPoints->isopoints, DisplayFunction->Identity],
		{}];
		
(*
	Put it all together in the Big Show. 
*)

If[shownumbers,
$outputNotebook=NotebookCreate[WindowTitle->"Output",
				WindowMargins-> {{0,Automatic},{Automatic,0}}, WindowSize->{8*72,2*72},
				Magnification-> 0.75, Background->Hue[Random[Real,{0,1}],.2,1]];
NotebookWrite[$outputNotebook, Cell[BoxData[ToBoxes[NumberForm[allpoints,16]]],"Output"]]];
	
Show[{xnulls, ynulls, isos, field, forwardplots, backwardplots, endpoints, startpoints, eqpoints},
	FilterOptions[Graphics,opts],
	FilterOptions[Graphics, Options[PhasePlot]]]
	
];  (* end of module *)

(** minsquaredistance function used in PhasePlot *)

	minsquaredistance[x_,y_]:=Min[Map[((x-#).(x-#))&,y]];

(** line clipping function used in PhasePlot **) 
	
	Attributes[pull]={HoldFirst};

	pull[z_, end_, xory_, k_, bounds_]:= With[
		{r=(bounds[[xory,k]]-z[[2end,xory]])/(z[[end,xory]]-z[[2end,xory]])},
		z[[end]]=r*z[[end]]+(1-r)*z[[2end]]];
											 (* Danke schoen, Herr Hartmut Wolf! *)
	clip[zIn_, bounds_]:=
	Module[{z=zIn},
	Scan[Function[end,
			If[z[[end,1]]<bounds[[1,1]], pull[z,end,1,1,bounds],
			If[z[[end,1]]>bounds[[1,-1]], pull[z,end,1,-1,bounds]]];
			If[z[[end,-1]]<bounds[[-1,1]], pull[z,end,-1,1,bounds],
			If[z[[end,-1]]>bounds[[-1,-1]], pull[z,end,-1,-1,bounds]]];],
			{1,-1}]; z];


(*** definition of DEPlot ***)

DEPlot[g_, {t_Symbol, t1_?NumericQ, t2_?NumericQ},
			{y_Symbol, y1_?NumericQ, y2_?NumericQ}, opts___Rule] :=
	Module[{ta, tb, x1, x2, seeds, plotrange},
	DEPlot::"tint"="Time interval must include zero.";
		If[t1<=0<=t2, {},
		Message[DEPlot::"tint"];Return[$Failed]];
		
		DEPlot::"num"="Slope function does not provide numerical values.";
		If[Not[NumericQ[g/.{t->0., y->N[y1]}]], Message[DEPlot::"num"];Return[$Failed]];

		maxcurves = MaxCurves /. {opts} /. Options[DEPlot];
		seeds = InitialPoints /. {opts} /. Options[DEPlot];
		plotrange = PlotRange /. {opts} /. Options[DEPlot];
		seeds=If[maxcurves===None || maxcurves===0 || maxcurves==={0}, None, seeds];
		seeds=Which[
      		seeds===Automatic && t1<=0<=t2,
      			Transpose[{{0,0,0,0},{y1,y2}.{{1,2,3,4},{4,3,2,1}}/5.}],
      		seeds===Automatic,
      			Transpose[{(t1+t2){1,1,1,1}/2.,{y1,y2}.{{1,2,3,4},{4,3,2,1}}/5.}],
     		 1==1,
      			seeds];
		ta=If[seeds===None, 0, Min[(seeds//Transpose)[[1]]]];
		tb=If[seeds===None, 0, Max[(seeds//Transpose)[[1]]]];
		{x1,x2}=
			Which[MatrixQ[plotrange],
					plotrange[[1]], 
				Head[plotrange]===List && plotrange[[2]]===Automatic,
					plotrange[[1]],
				1==1,
					{t1,t2}
				];
		PhasePlot[{1, g}, {t, t1-tb, t2-ta}, {t,x1,x2}, {y,y1,y2}, InitialPoints->seeds, opts]
	];

   
    
(* definition of TimeStatePlot *)

TimeStatePlot[{xsoln_, ysoln_}, {t_Symbol, tmin_?NumericQ, tmax_?NumericQ}, opts___Rule] := 
   Module[{curve, curvex, curvey, curvez, xshadow, yshadow, zshadow, colors,
   			shadows, style, trip, tplotrange, t1, t2, tt},
   		
		TimeStatePlot::"num"="Functions do not provide numerical values.";
		If[And@@NumericQ/@({xsoln,ysoln}/.{t->0.}),{}, Message[TimeStatePlot::"num"];Return[$Failed]];   		

		colors = ShadowColors /. {opts} /. Options[TimeStatePlot]; 
		shadows = ShadowPositions /. {opts} /. Options[TimeStatePlot]; 
		style = PlotStyle /. {opts} /. Options[TimeStatePlot]; 
		style=If[style===Automatic, {}, style];
		
		tplotrange= PlotRange /. {opts} /.{PlotRange->{tmin,tmax}};
		tplotrange= If[Length[tplotrange]==3, tplotrange[[1]], tplotrange];
		tplotrange= If[tplotrange===Automatic,{tmin,tmax}, tplotrange];
		{t1,t2}=tplotrange;
		tt= (t2(t-tmin)-t1(t-tmax))/(tmax-tmin);
		
	trip := {t, xsoln, ysoln};

	curve := ParametricPlot3D[Evaluate[{{tt, 0, 0}, Append[trip, Flatten[style]]}], 
       {t, tmin, tmax}, DisplayFunction-> Identity,
       Evaluate[FilterOptions[ParametricPlot3D,opts]],
       Evaluate[FilterOptions[ParametricPlot3D,Options[TimeStatePlot]]]]; 
     
	curvex := ParametricPlot3D[Evaluate[Append[trip, Flatten[{style, colors[[1]]}]]],
		{t, tmin, tmax}, DisplayFunction-> Identity,
		Evaluate[FilterOptions[ParametricPlot3D,opts]],
		Evaluate[FilterOptions[ParametricPlot3D,Options[TimeStatePlot]]]]; 
	curvey := ParametricPlot3D[Evaluate[{{tt, 0, 0}, Append[trip, Flatten[{style, colors[[2]]}]]}],
		{t, tmin, tmax}, DisplayFunction-> Identity,
		Evaluate[FilterOptions[ParametricPlot3D,opts]],
		Evaluate[FilterOptions[ParametricPlot3D,Options[TimeStatePlot]]]]; 
	curvez := ParametricPlot3D[Evaluate[{{tt, 0, 0}, Append[trip, Flatten[{style, colors[[3]]}]]}],
		{t, tmin, tmax}, DisplayFunction-> Identity,
		Evaluate[FilterOptions[ParametricPlot3D,opts]],
		Evaluate[FilterOptions[ParametricPlot3D,Options[TimeStatePlot]]]]; 
     
	xshadow := Shadow[curvex, XShadowPosition-> shadows[[1]], ZShadow-> False, YShadow-> False,
		DisplayFunction-> Identity]; 
	yshadow := Shadow[curvey, YShadowPosition-> shadows[[2]], ZShadow-> False, XShadow-> False,
		DisplayFunction-> Identity]; 
	zshadow := Shadow[curvez, ZShadowPosition-> shadows[[3]], XShadow-> False, YShadow-> False,
		 DisplayFunction-> Identity]; 

	Show[xshadow, yshadow, zshadow, curve,
		 FilterOptions[Graphics3D,opts],
		 FilterOptions[Graphics3D,Options[TimeStatePlot]]
		 ]

]; 

(*** Definition of PoincareTimeSection ***)

PoincareTimeSection[{f_, g_}, {t_Symbol, t0_?NumericQ, tend_?NumericQ, dt_?NumericQ},
				{x_Symbol, x0_?NumericQ}, {y_Symbol, y0_?NumericQ}, opts___Rule] := 
  Module[{xsoln, ysoln, maxsteps}, 
  
  PoincareTimeSection::"num"="Vector field does not provide numerical values.";
  If[And@@NumericQ/@({f,g}/.{t->0.,x->N[x0],y->N[y0]}),{}, Message[PoincareTimeSection::"num"];Return[$Failed]];
  
  maxsteps= MaxSteps/.{opts}/.{MaxSteps->100(tend-t0)};
  {xsoln, ysoln} = {x, y} /.
    NDSolve[{x'[t] == (f /. {x-> x[t], y-> y[t]}), y'[t] == (g /. {x-> x[t], y-> y[t]}),
    x[0] == x0, y[0] == y0}, {x, y}, {t, t0, tend}, MaxSteps->maxsteps,
    FilterOptions[NDSolve,opts], FilterOptions[NDSolve, Options[PoincareTimeSection]]][[1]];
    ListPlot[Table[{xsoln[t], ysoln[t]}, {t, t0, tend, dt}],
    FilterOptions[ListPlot,opts],
    FilterOptions[ListPlot, Options[PoincareTimeSection]]]
  ];
    

(*** Definition of ViewProjections ***)
    
ViewProjections[{f_, g_, h_}, {t_Symbol, t0_?NumericQ, tmax_?NumericQ}, {x_, y_, z_}, opts___Rule] := 
  Module[
  {xyProj, xzProj, yzProj, xx, yy, zz, outerframe, outerframestyle, optpp}, 
  
    ViewProjections::"num"="Functions not provide numerical values.";
    If[And@@NumericQ/@({f,g,h}/.{t->N[t0]}),{}, Message[ViewProjections::"num"];Return[$Failed]];

	xx = StyleForm[x, FontSlant-> "Italic"];
	yy = StyleForm[y, FontSlant-> "Italic"]; 
	zz = StyleForm[z, FontSlant-> "Italic"];
	
	optpp = FilterOptions[ParametricPlot, opts];
   
	outerframe = OuterFrame /.{opts} /.{OuterFrame-> False}; 
	outerframe= If[outerframe=!=True, False, True];
	outerframestyle = OuterFrameStyle /.{opts} /.{OuterFrameStyle-> None};
	
			
	xyProj = ParametricPlot[Evaluate[{f,g}], {t, t0, tmax}, PlotLabel-> None, 
		AxesLabel-> {xx, yy}, DisplayFunction-> Identity, Evaluate[optpp]]; 
	xzProj = ParametricPlot[Evaluate[{f,h}], {t, t0, tmax}, PlotLabel-> None, 
		AxesLabel-> {xx, zz}, DisplayFunction-> Identity, Evaluate[optpp]]; 
	yzProj = ParametricPlot[Evaluate[{g,h}], {t, t0, tmax}, PlotLabel-> None, 
		AxesLabel-> {yy, zz}, DisplayFunction-> Identity, Evaluate[optpp]];
      
    Show[GraphicsArray[{xyProj, xzProj, yzProj}],
    	Frame->outerframe, FrameTicks->None,
    	FrameStyle->outerframestyle, Axes->False, AspectRatio->Automatic,
    	FilterOptions[GraphicsArray, opts],
    	FilterOptions[GraphicsArray, Options[ViewProjections]]]
  ];
    
    
(*** Definition of PlotImplicit ***)

PlotImplicit[f_, {x_Symbol, xmin_?NumericQ, xmax_?NumericQ},
			{y_Symbol, ymin_?NumericQ, ymax_?NumericQ}, opts___Rule] := 
  Module[{curves, density, plotpoints, showdensity},

	plotpoints= PlotPoints/. {opts}/. Options[PlotImplicit];
    showdensity = ShowDensity /. {opts} /. Options[PlotImplicit]; 
    showdensity = If[showdensity =!= True && showdensity =!= False, False, showdensity];
	
   	curves = ContourPlot[Evaluate[f], {x, xmin, xmax}, {y, ymin, ymax},
   		DisplayFunction-> Identity,
   		Evaluate[FilterOptions[ContourPlot,opts]],
     	Evaluate[FilterOptions[ContourPlot,Options[PlotImplicit]]]];
	density =
		If[showdensity, 
			DensityPlot[Evaluate[f], {x, xmin, xmax}, {y, ymin, ymax}, 
      			DisplayFunction-> Identity,
				Evaluate[FilterOptions[DensityPlot,opts]],
				PlotPoints-> plotpoints+25,
				Evaluate[FilterOptions[DensityPlot,Options[PlotImplicit]]]],
		{}];
	 
    Show[density, curves,
    	FilterOptions[Graphics, opts],
    	FilterOptions[Graphics, Options[PlotImplicit]]];
]


End[]

EndPackage[]