
(* :Package Version: 1.10.20 *)

(* :Copyright: Copyright 2002, Selwyn Hollis *)

(* :Name: DiffEqs`FourierTools` *)
 
(* :Author: Selwyn Hollis *)


BeginPackage["DiffEqs`FourierTools`"]

FourierSum::usage ="FourierSum[f[x], {x,-L,L}, m] returns the first m terms \
of the Fourier series of f[x] on [-L,L]."

NFourierSum::usage ="NFourierSum[f[x], {x,-L,L}, m] returns the first m terms \
of the Fourier series of f[x] on [-L,L] with numerical coefficients."

FourierCosSum::usage ="FourierCosSum[f[x], {x,0,L}, m] returns the first m \
terms of the Fourier cosine series of f[x] on [0,L]."

FourierSinSum::usage ="FourierSinSum[f[x], {x,0,L}, m] returns the first m \
terms of the Fourier sine series of f[x] on [0,L]."

NFourierCosSum::usage ="NFourierCosSum[f[x], {x,0,L}, m] returns the first m \
terms of the Fourier cosine series of f[x] on [0,L] with numerical \
coefficients."

NFourierSinSum::usage ="NFourierSinSum[f[x], {x,0,L}, m] returns the first m \
terms of the Fourier sine series of f[x] on [0,L] with numerical coefficients."

Begin["`Private`"]

Off[NIntegrate::ploss, NIntegrate::ncvb];

FourierSum[f_, {x_Symbol, minusL_, L_}, m_Integer] :=
     If[minusL === -L, Module[{an,bn},
     an=Integrate[f*Cos[n*Pi*x/L], {x, -L, L}];
     bn=Integrate[f*Sin[n*Pi*x/L], {x, -L, L}];
    (1/(2*L))*Integrate[f, {x, -L, L}] +
    (1/L)*Sum[an*Cos[(n*Pi*x)/L] + bn*Sin[(n*Pi*x)/L], {n, 1, m}]],
       Print["Interval must be of the form [-L,L]."];{}]; 
       
NFourierSum[f_, {x_Symbol, minusL_?NumericQ, L_?NumericQ}, m_Integer] := 
If[minusL == -L, 
    Chop[(1/(2*L))*NIntegrate[f, {x, -L, L}] + (1/L)*
       Sum[NIntegrate[f*Cos[n*Pi*x/L], {x, -L, L}]*Cos[n*Pi*x/L] + 
         NIntegrate[f*Sin[n*Pi*x/L], {x, -L, L}]*Sin[n*Pi*x/L], 
        {n, 1, m}]],
    Print["Interval must be of the form [-L,L]."];{}]; 
        
FourierCosSum[f_, {x_Symbol, 0|0., L_}, m_Integer] :=
Module[{an},
an=Integrate[f*Cos[n*Pi*x/L],{x, 0, L}];
   (1/L)*Integrate[f,{x, 0, L}] + 
    (2/L)*Sum[an*Cos[n*Pi*x/L],{n, 1, m}]]; 
      
FourierSinSum[f_, {x_Symbol, 0|0., L_}, m_Integer] := 
Module[{bn},
bn=Integrate[f*Sin[n*Pi*x/L],{x, 0, L}];
   (2/L)*Sum[bn*Sin[n*Pi*x/L], {n, 1, m}]]; 
     
NFourierCosSum[f_, {x_Symbol, 0|0., L_?NumericQ}, m_Integer] := 
   Chop[(1/L)*NIntegrate[f, {x, 0, L}] + 
     (2/L)*Sum[NIntegrate[f*Cos[n*Pi*x/L], {x, 0, L}]*Cos[n*Pi*x/L], 
       {n, 1, m}]]; 
       
NFourierSinSum[f_, {x_Symbol, 0|0., L_?NumericQ}, m_Integer] := 
   Chop[(2/L)*Sum[NIntegrate[f*Sin[n*Pi*x/L], {x, 0, L}]*
       Sin[n*Pi*x/L], {n, 1, m}]]; 
       
       
End[]

EndPackage[]
