(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     38629,       1324]*)
(*NotebookOutlinePosition[     39374,       1350]*)
(*  CellTagsIndexPosition[     39330,       1346]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  "DiffEqs",
  StyleBox["`", "MB"],
  "DEGraphics",
  StyleBox["`", "MB"]
}], "Subsection"],

Cell["\<\
This package provides the following functions for plotting \
solutions of differential equations. \
\>", "Text"],

Cell[TextData[{
  StyleBox["DEPlot[",
    FontFamily->"Courier"],
  Cell[BoxData[
      \(TraditionalForm\`f[t, \ y]\)]],
  ", {",
  StyleBox["t",
    FontSlant->"Italic"],
  ", ",
  StyleBox["tmin",
    FontSlant->"Italic"],
  ", ",
  StyleBox["tmax",
    FontSlant->"Italic"],
  "}, {",
  StyleBox["y",
    FontSlant->"Italic"],
  ", ",
  StyleBox["ymin",
    FontSlant->"Italic"],
  ", ",
  StyleBox["ymax",
    FontSlant->"Italic"],
  "}",
  StyleBox["] ",
    FontFamily->"Courier"],
  "creates a plot showing the direction field and \n\tsolution curves for the \
differential equation ",
  Cell[BoxData[
      \(TraditionalForm\`y\^\[Prime] = f(t, \ y)\)]],
  ".\n\t\n",
  StyleBox["PhasePlot[",
    FontFamily->"Courier"],
  "{",
  Cell[BoxData[
      \(TraditionalForm\`f[x, \ y], \ g[x, \ y]\)]],
  "}, {",
  StyleBox["t",
    FontSlant->"Italic"],
  ", ",
  StyleBox["tmin",
    FontSlant->"Italic"],
  ", ",
  StyleBox["tmax",
    FontSlant->"Italic"],
  "}, {",
  StyleBox["x",
    FontSlant->"Italic"],
  ", ",
  StyleBox["xmin",
    FontSlant->"Italic"],
  ", ",
  StyleBox["xmax",
    FontSlant->"Italic"],
  "}, {",
  StyleBox["y",
    FontSlant->"Italic"],
  ", ",
  StyleBox["ymin",
    FontSlant->"Italic"],
  ", ",
  StyleBox["ymax",
    FontSlant->"Italic"],
  "}",
  StyleBox["]\n",
    FontFamily->"Courier"],
  "\tcreates a plot showing the direction field and solution curves for the \
autonomous system\n\t",
  Cell[BoxData[
      \(TraditionalForm\`x\^\[Prime] = f(x, \ y), \ \ y\^\[Prime] = 
        g(x, \ y)\)]],
  ".\n\n",
  StyleBox["PhasePlot[",
    FontFamily->"Courier"],
  "{",
  Cell[BoxData[
      \(TraditionalForm\`f[t, \ x, \ y], \ g[t, \ x, \ y]\)]],
  "}, {",
  StyleBox["t",
    FontSlant->"Italic"],
  ", ",
  StyleBox["tmin",
    FontSlant->"Italic"],
  ", ",
  StyleBox["tmax",
    FontSlant->"Italic"],
  "}, {",
  StyleBox["x",
    FontSlant->"Italic"],
  ", ",
  StyleBox["xmin",
    FontSlant->"Italic"],
  ", ",
  StyleBox["xmax",
    FontSlant->"Italic"],
  "}, {",
  StyleBox["y",
    FontSlant->"Italic"],
  ", ",
  StyleBox["ymin",
    FontSlant->"Italic"],
  ", ",
  StyleBox["ymax",
    FontSlant->"Italic"],
  "}",
  StyleBox["]",
    FontFamily->"Courier"],
  "\n\tcreates a plot showing solution curves for the system ",
  Cell[BoxData[
      \(TraditionalForm\`x\^\[Prime] = f(t, \ x, \ y), \ \ y\^\[Prime] = 
        g(t, \ x, \ y)\)]],
  ".\n\t\n",
  StyleBox["NDPlot[",
    FontFamily->"Courier"],
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox["eqns",
            FontSlant->"Italic"], ","}], TraditionalForm]]],
  " ",
  StyleBox["fns",
    FontSlant->"Italic"],
  ", {",
  StyleBox["t",
    FontSlant->"Italic"],
  ", ",
  StyleBox["tmin",
    FontSlant->"Italic"],
  ", ",
  StyleBox["tmax",
    FontSlant->"Italic"],
  "}",
  StyleBox["]\n\t ",
    FontFamily->"Courier"],
  "uses ",
  StyleBox["NDSolve",
    FontFamily->"Courier"],
  " to compute a numerical solution of a system of (up to 3) first-order \t\n\
\tdifferential equations and plots the solution. Returns {",
  StyleBox["solution",
    FontSlant->"Italic"],
  ", ",
  StyleBox["-Graphics-",
    FontFamily->"Courier",
    FontSize->11],
  "}.\n\t\n",
  StyleBox["PoincareTimeSection[",
    FontFamily->"Courier"],
  "{",
  Cell[BoxData[
      \(TraditionalForm\`f[t, \ x, \ y], \ g[t, \ x, \ y]\)]],
  "}, {",
  StyleBox["t",
    FontSlant->"Italic"],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`t\_0\)]],
  ", ",
  StyleBox["tmax, ",
    FontSlant->"Italic"],
  "\[CapitalDelta]",
  StyleBox["\[VeryThinSpace]t",
    FontSlant->"Italic"],
  "}, {",
  StyleBox["x",
    FontSlant->"Italic"],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`x\_0\)]],
  "}, {",
  StyleBox["y",
    FontSlant->"Italic"],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`y\_0\)]],
  "}",
  StyleBox["] ",
    FontFamily->"Courier"],
  "creates a \n\tPoincar\[EAcute] time section plot of the solution of  ",
  Cell[BoxData[
      \(TraditionalForm\`x\^\[Prime] = f(t, \ x, \ y), \ 
      x(t\_0) = x\_0, \)]],
  "  ",
  Cell[BoxData[
      \(TraditionalForm\`y\^\[Prime] = g(t, \ \[VeryThinSpace]x, \ y), \ 
      y(t\_0) = y\_0\)]],
  ".\n\n",
  StyleBox["ViewProjections[",
    FontFamily->"Courier"],
  "{",
  Cell[BoxData[
      \(TraditionalForm\`f[t], \ g[t], \ h[t]\)]],
  "}, {",
  StyleBox["t",
    FontSlant->"Italic"],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`t\_0\)]],
  ", ",
  StyleBox["tmax",
    FontSlant->"Italic"],
  "}, {",
  Cell[BoxData[
      \(TraditionalForm\`x, y, z\)]],
  "}",
  StyleBox["] ",
    FontFamily->"Courier"],
  "creates a GraphicsArray of \n\tprojections of the curve ",
  Cell[BoxData[
      \(TraditionalForm\`\((f(t), \ g(t), \ h(t))\)\)]],
  " onto the ",
  Cell[BoxData[
      \(TraditionalForm\`x\[ThinSpace]y\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`x\[ThinSpace]z\)]],
  ", and ",
  Cell[BoxData[
      \(TraditionalForm\`y\[ThinSpace]z\)]],
  " coordinate planes.\n\n",
  StyleBox["PlotImplicit[",
    FontFamily->"Courier"],
  Cell[BoxData[
      \(TraditionalForm\`F[t, \ y]\)]],
  ", {",
  StyleBox["t",
    FontSlant->"Italic"],
  ", ",
  StyleBox["tmin",
    FontSlant->"Italic"],
  ", ",
  StyleBox["tmax",
    FontSlant->"Italic"],
  "}, {",
  StyleBox["y",
    FontSlant->"Italic"],
  ", ",
  StyleBox["ymin",
    FontSlant->"Italic"],
  ", ",
  StyleBox["ymax",
    FontSlant->"Italic"],
  "}",
  StyleBox["] ",
    FontFamily->"Courier"],
  "creates a contour plot of the function ",
  Cell[BoxData[
      \(TraditionalForm\`F\)]],
  ".\n\n",
  StyleBox["TimeStatePlot[",
    FontFamily->"Courier"],
  "{",
  Cell[BoxData[
      \(TraditionalForm\`x[t], \ y[t]\)]],
  "}, {",
  StyleBox["t",
    FontSlant->"Italic"],
  ", ",
  StyleBox["tmin",
    FontSlant->"Italic"],
  ", ",
  StyleBox["tmax",
    FontSlant->"Italic"],
  "}, {",
  StyleBox["y",
    FontSlant->"Italic"],
  ", ",
  StyleBox["ymin",
    FontSlant->"Italic"],
  ", ",
  StyleBox["ymax",
    FontSlant->"Italic"],
  "}",
  StyleBox["] ",
    FontFamily->"Courier"],
  "creates a 3D plot of the \[OpenCurlyDoubleQuote]time-state \n\ttrajectory\
\[CloseCurlyDoubleQuote] ",
  Cell[BoxData[
      \(TraditionalForm\`\((t, \[ThinSpace]x(t), \[ThinSpace]y(t))\)\)]],
  " with projections onto the ",
  StyleBox["t\[VeryThinSpace]x",
    FontSlant->"Italic"],
  ", ",
  StyleBox["t\[VeryThinSpace]y",
    FontSlant->"Italic"],
  ", and ",
  StyleBox["x\[VeryThinSpace]y",
    FontSlant->"Italic"],
  " planes."
}], "DefinitionBox",
  CellFrame->True,
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic,
  ParagraphSpacing->{0, 0}],

Cell["Plotting solutions of differential equations. ", "Caption"],

Cell["This loads the package.", "MathCaption"],

Cell["<<DiffEqs`DEGraphics`", "Input",
  CellLabel->"In[56]:="],

Cell[CellGroupData[{

Cell["DEPlot", "SubsectionIcon"],

Cell[TextData[{
  "Here are a few examples of the use of ",
  StyleBox["DEPlot",
    FontFamily->"Courier"],
  ". Each of these uses the differential equation ",
  Cell[BoxData[
      \(TraditionalForm\`y\^\[Prime] = y\ \((cos(t) - y)\)\)]],
  ". "
}], "MathCaption"],

Cell[BoxData[
    \(DEPlot[y\ \((Cos[t] - y)\), {t, \(-2\), 4}, {y, \(-1\), 2}]\)], "Input",\

  CellLabel->"In[57]:="],

Cell[BoxData[
    \(DEPlot[y\ \((Cos[t] - y)\), {t, \(-2\), 4}, {y, \(-1\), 2}, 
      MaxCurves \[Rule] 3, Gap \[Rule]  .03]\)], "Input",
  CellLabel->"In[58]:="],

Cell[BoxData[
    \(DEPlot[y\ \((Cos[t] - y)\), {t, \(-2\), 4}, {y, \(-1\), 2}, 
      InitialPoints \[Rule] {{0, 1}, {2, 1}}, 
      MaxCurves \[Rule] {3, 1}]\)], "Input",
  CellLabel->"In[59]:="],

Cell[BoxData[
    \(DEPlot[y\ \((Cos[t] - y)\), {t, \(-2\), 4}, {y, \(-1\), 2}, 
      Isoclines \[Rule] {0, \(-1\)}]\)], "Input",
  CellLabel->"In[60]:="],

Cell[TextData[{
  StyleBox["DEPlot",
    FontFamily->"Courier"],
  " accepts any of the options of ",
  StyleBox["ListPlot",
    FontFamily->"Courier"],
  ", as well as the following:"
}], "MathCaption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic],

Cell[BoxData[
    StyleBox[
      FrameBox[
        StyleBox[GridBox[{
              {
                StyleBox[\(option\ name\),
                  FontFamily->"Times",
                  FontSlant->"Italic"], 
                StyleBox[\(default\ value\),
                  "TI"], 
                StyleBox["\[Null]",
                  FontFamily->"Times"]},
              {"InitialPoints", "Automatic", 
                
                StyleBox[\(A\ list\ of\ initial\ points\ for\ generated\ \
\(\(curves\)\(.\)\)\),
                  FontFamily->"Times"]},
              {"MaxCurves", "1", 
                
                StyleBox[\(\(The\ number\ of\ curves\ to\ generate\ from\ \
each\ initial\ \(\(point\)\(.\)\)\)\ \(May\ be\ a\ single\ number\ or\ a\ \
list\ of\ numbers\ corresponding\ to\ the\ list\ of\ initial\ \(\(points\)\(.\
\)\)\)\),
                  FontFamily->"Times"]},
              {"Gap", "0.1", 
                
                StyleBox[\(\("Controls the spacing between automatically \
generated starting points"\)\(.\)\),
                  FontFamily->"Times"]}
              },
            ColumnAlignments->{Left}],
          GridBoxOptions->{RowLines->{True, False}}]],
      "3ColumnBox"]], "Text",
  CellFrame->False,
  LineSpacing->{1.21, 0},
  GridBoxOptions->{RowSpacings->1,
  ColumnSpacings->0.8,
  ColumnAlignments->{Left},
  RowsEqual->False,
  ColumnsEqual->False}],

Cell[TextData[{
  "Curve options for ",
  StyleBox["DEPlot", "MR"],
  "."
}], "Caption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic],

Cell[TextData[{
  "With ",
  StyleBox["InitialPoints\[ThinSpace]\[Rule]\[ThinSpace]Automatic",
    FontFamily->"Courier"],
  ", ",
  StyleBox["DEPlot",
    FontFamily->"Courier"],
  " uses four uniformly spaced initial points to draw four curves. With ",
  StyleBox["InitialPoints\[ThinSpace]\[Rule]\[ThinSpace]None",
    FontFamily->"Courier"],
  " or ",
  StyleBox["MaxCurves\[ThinSpace]\[Rule]\[ThinSpace]0",
    FontFamily->"Courier"],
  ", no curves are drawn. If ",
  StyleBox["MaxCurves\[ThinSpace]\[Rule]",
    FontFamily->"Courier"],
  " ",
  StyleBox["n",
    FontSlant->"Italic"],
  ", where ",
  Cell[BoxData[
      \(TraditionalForm\`n > 0\)]],
  ", then, for each initial point, ",
  StyleBox["DEPlot",
    FontFamily->"Courier"],
  " attempts to generate ",
  Cell[BoxData[
      \(TraditionalForm\`n - 1\)]],
  " additional starting points if ",
  StyleBox["n",
    FontSlant->"Italic"],
  " is odd and ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " additional starting points if ",
  StyleBox["n",
    FontSlant->"Italic"],
  " is even. This is done by taking steps orthogonal to the direction field."
}], "Text"],

Cell[BoxData[
    FormBox[
      StyleBox[
        FrameBox[
          StyleBox[GridBox[{
                {
                  StyleBox[\(option\ name\),
                    FontFamily->"Times",
                    FontSlant->"Italic"], 
                  StyleBox[\(default\ value\),
                    "TI"], "\[Null]"},
                {
                  StyleBox["Arrows",
                    "MR",
                    FontFamily->"Courier"], 
                  StyleBox[\({11, 9}\),
                    
                    FontFamily->
                      "Courier"], \(\("The number of arrows to draw in each \
direction"\)\(.\)\)},
                {
                  StyleBox["ArrowColor",
                    FontFamily->"Courier"], 
                  StyleBox[\(RGBColor[0,  .6,  .6]\),
                    
                    FontFamily->
                      "Courier"], \(Color\ of\ the\ \(\(arrows\)\(.\)\)\)},
                {
                  StyleBox["ScaleArrows",
                    FontFamily->"Courier"], 
                  StyleBox["1",
                    
                    FontFamily->
                      "Courier"], \(\(\(Controls\ the\ arrows\)\(\
\[CloseCurlyQuote]\)\)\ \(\(length\)\(.\)\)\)}
                },
              ColumnAlignments->{Left}],
            GridBoxOptions->{RowLines->{True, False}}]],
        "3ColumnBox"], TraditionalForm]], "Text",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic,
  LineSpacing->{1.1, 0},
  GridBoxOptions->{RowSpacings->1.25}],

Cell[TextData[{
  "Arrow style options for ",
  StyleBox["DEPlot", "MR"],
  "."
}], "Caption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic],

Cell[BoxData[
    FormBox[
      StyleBox[
        FrameBox[
          StyleBox[GridBox[{
                {
                  StyleBox[\(option\ name\),
                    FontFamily->"Times",
                    FontSlant->"Italic"], 
                  StyleBox[\(default\ value\),
                    "TI"], "\[Null]"},
                {
                  StyleBox["Isoclines",
                    FontFamily->"Courier"], 
                  StyleBox["False",
                    FontFamily->"Courier"], 
                  RowBox[{
                    StyleBox["Whether",
                      FontFamily->"Times"], 
                    StyleBox[" ",
                      FontFamily->"Times"], 
                    StyleBox["to",
                      FontFamily->"Times"], 
                    StyleBox[" ",
                      FontFamily->"Times"], 
                    StyleBox["draw",
                      FontFamily->"Times"], 
                    StyleBox[" ",
                      FontFamily->"Times"], 
                    StyleBox[\(isoclines . \ May\),
                      FontFamily->"Times"], 
                    StyleBox[" ",
                      FontFamily->"Times"], 
                    StyleBox["also",
                      FontFamily->"Times"], 
                    StyleBox[" ",
                      FontFamily->"Times"], 
                    StyleBox["be",
                      FontFamily->"Times"], 
                    StyleBox[" ",
                      FontFamily->"Times"], 
                    StyleBox["a",
                      FontFamily->"Times",
                      FontSlant->"Plain"], 
                    StyleBox[" ",
                      FontFamily->"Times"], 
                    StyleBox["list",
                      FontFamily->"Times"], 
                    StyleBox[" ",
                      FontFamily->"Times"], 
                    StyleBox["of",
                      FontFamily->"Times"], 
                    StyleBox[" ",
                      FontFamily->"Times"], 
                    StyleBox["slope",
                      FontFamily->"Times"], 
                    StyleBox[" ",
                      FontFamily->"Times"], 
                    StyleBox[\(\(values\)\(.\)\),
                      FontFamily->"Times"]}]},
                {
                  StyleBox["ShowStartPoints",
                    "MR",
                    FontFamily->"Courier"], 
                  StyleBox["False",
                    FontFamily->"Courier"], 
                  
                  StyleBox[\(Whether\ to\ show\ the\ initial\ point\ from\ \
which\ each\ curve\ is\ \(\(drawn\)\(.\)\)\),
                    FontFamily->"Times"]},
                {
                  StyleBox["ShowEquilibriumPoints",
                    FontFamily->"Courier"], 
                  StyleBox["False",
                    FontFamily->"Courier"], 
                  
                  StyleBox[\(Whether\ to\ show\ equilibrium\ \
points/\(\(solutions\)\(.\)\)\),
                    FontFamily->"Times"]},
                {
                  StyleBox["ShowField",
                    FontFamily->"Courier"], 
                  StyleBox["True",
                    
                    FontFamily->
                      "Courier"], \(\("Whether to show the direction field"\)\
\(.\)\)},
                {
                  StyleBox["UseVectorField",
                    FontFamily->"Courier"], 
                  StyleBox["False",
                    FontFamily->"Courier"], 
                  
                  StyleBox[\(Whether\ to\ show\ the\ vector\ field\ rather\ \
than\ the\ direction\ \(\(field\)\(.\)\)\),
                    FontFamily->"Times"]}
                },
              ColumnAlignments->{Left}],
            GridBoxOptions->{RowLines->{True, False}}]],
        "3ColumnBox"], TraditionalForm]], "Text",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic,
  LineSpacing->{1.1, 0},
  GridBoxOptions->{RowSpacings->1.25}],

Cell[TextData[{
  "Display options for ",
  StyleBox["DEPlot", "MR"],
  "."
}], "Caption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic],

Cell[BoxData[
    FormBox[
      StyleBox[
        FrameBox[
          StyleBox[GridBox[{
                {
                  StyleBox[\(option\ name\),
                    FontFamily->"Times",
                    FontSlant->"Italic"], 
                  StyleBox[\(default\ value\),
                    "TI"], "\[Null]"},
                {
                  StyleBox["StepSize",
                    FontFamily->"Courier"], 
                  StyleBox["0.1",
                    
                    FontFamily->
                      "Courier"], \(\(The\ stepsize\ for\ the\ numerical\)\ \
\(\(method\)\(.\)\)\)},
                {
                  StyleBox["Method",
                    FontFamily->"Courier"], 
                  StyleBox["RungeKutta",
                    FontFamily->"Courier"], 
                  RowBox[{
                    RowBox[{
                    "The", " ", "numerical", " ", "method", " ", "to", 
                      RowBox[{"use", ":", " ", 
                        StyleBox["Euler",
                          FontFamily->"Courier"]}]}], ",", " ", 
                    StyleBox["Heun",
                      FontFamily->"Courier"], ",", " ", 
                    StyleBox["RungeKutta",
                      FontFamily->"Courier"], ",", " ", 
                    StyleBox["DormandPrince",
                      FontFamily->"Courier"], ",", " ", 
                    RowBox[{"or", " ", 
                      StyleBox["Automatic",
                        FontFamily->"Courier"]}]}]}
                },
              ColumnAlignments->{Left}],
            GridBoxOptions->{RowLines->{True, False}}]],
        "3ColumnBox"], TraditionalForm]], "Text",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic,
  LineSpacing->{1.1, 0},
  GridBoxOptions->{RowSpacings->1.25}],

Cell[TextData[{
  "Numerical options for ",
  StyleBox["DEPlot", "MR"],
  "."
}], "Caption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic],

Cell[TextData[{
  "Standard (nonadaptive) fourth-order Runge-Kutta is the default numerical \
method. ",
  StyleBox["Heun",
    FontFamily->"Courier"],
  " is the second-order method that is often called the improved, or \
modified,  Euler method. With ",
  StyleBox["Method\[ThinSpace]\[RightArrow]\[ThinSpace]DormandPrince",
    FontFamily->"Courier"],
  ",  the eighth-order (nonadaptive) Dormand-Prince method is used.  ",
  StyleBox["Method\[ThinSpace]\[RightArrow]\[ThinSpace]Automatic",
    FontFamily->"Courier"],
  " causes ",
  StyleBox["DEPlot",
    FontFamily->"Courier"],
  " to use ",
  StyleBox["NDSolve",
    FontFamily->"Courier"],
  " successively over intervals ",
  Cell[BoxData[
      \(TraditionalForm\`\([t, \ t + h]\)\)]],
  ", where ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(h\)\(=\)\)\)]],
  " ",
  StyleBox["StepSize",
    FontFamily->"Courier"],
  ". Thus ",
  StyleBox["StepSize",
    FontFamily->"Courier"],
  " does not affect the accuracy in this case\[LongDash]only the smoothness \
of the curve drawn.  ",
  StyleBox["Method\[ThinSpace]\[RightArrow]\[ThinSpace]Automatic",
    FontFamily->"Courier"],
  " is generally much slower at a given stepsize, but for difficult (i.e., \
stiff) equations, may give much better results. When ",
  StyleBox["Method\[ThinSpace]\[RightArrow]\[ThinSpace]Automatic",
    FontFamily->"Courier"],
  ", any of the options of ",
  StyleBox["NDSolve",
    FontFamily->"Courier"],
  " can be used."
}], "Text",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic],

Cell[TextData[{
  "This illustrates the ",
  StyleBox["ShowEquilibriumPoints",
    FontFamily->"Courier"],
  " option."
}], "MathCaption"],

Cell[BoxData[
    \(DEPlot[y\ \((2 - y)\) -  .75, {t, \(-2\), 4}, {y, 0, 2}, 
      ShowEquilibriumPoints \[Rule] True]\)], "Input",
  CellLabel->"In[61]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["PhasePlot", "SubsectionIcon"],

Cell[TextData[{
  "Here is an example of the use of ",
  StyleBox["PhasePlot",
    FontFamily->"Courier"],
  ", with default value of ",
  StyleBox["InitialPoints",
    FontFamily->"Courier"],
  ". The system is ",
  Cell[BoxData[
      \(TraditionalForm\`x\^\[Prime] = x\^2 + y\^2 - 1, \ \ y\^\[Prime] = 
        x\^2 + y\^2 - 4\)]],
  "."
}], "MathCaption"],

Cell[BoxData[
    \(PhasePlot[{x\^2 + y\^2 - 1, x\^2 + y\^2 - 4}, {t, \(-4\), 
        4}, {x, \(-3\), 3}, {y, \(-3\), 3}, ShowNullclines \[Rule] True, 
      AspectRatio \[Rule] Automatic]\)], "Input",
  CellLabel->"In[62]:="],

Cell[TextData[{
  "No direction field is drawn if the vector field depends on ",
  StyleBox["t",
    FontSlant->"Italic"],
  "."
}], "MathCaption"],

Cell[BoxData[
    \(PhasePlot[{y, \(-t\)\ x}, {t, \(-1\), 10}, {x, \(-2\), 2}, {y, \(-2\), 
        2}]\)], "Input",
  CellLabel->"In[63]:="],

Cell["\<\
This uses the Fitzhugh-Nagumo equations and a single initial point.\
\
\>", "MathCaption"],

Cell[BoxData[
    \(PhasePlot[{3\ y - x\ \((x\^2 - 3)\) - 1.01, 
        1\/30\ \((7 - 10\ x - 8\ y)\)}, {t, 0, 40}, {x, \(-2.5\), 
        2.5}, {y, \(-1\), 1.5}, InitialPoints \[Rule] {{1, \(- .5\)}}, 
      ShowNullclines \[Rule] True, 
      ShowEquilibriumPoints \[Rule] True]\)], "Input",
  CellLabel->"In[64]:="],

Cell[TextData[{
  "Options for ",
  StyleBox["PhasePlot",
    FontFamily->"Courier"],
  " are the same as for ",
  StyleBox["DEPlot",
    FontFamily->"Courier"],
  ", except that ",
  StyleBox["Isoclines",
    FontFamily->"Courier"],
  " is replaced by ",
  StyleBox["ShowNullclines",
    FontFamily->"Courier"],
  "."
}], "MathCaption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic],

Cell[BoxData[
    FormBox[
      StyleBox[
        FrameBox[
          StyleBox[GridBox[{
                {
                  StyleBox[\(option\ name\),
                    FontFamily->"Times",
                    FontSlant->"Italic"], 
                  StyleBox[\(default\ value\),
                    "TI"], " "},
                {
                  StyleBox["ShowNullclines",
                    FontFamily->"Courier"], 
                  StyleBox["False",
                    
                    FontFamily->
                      "Courier"], \(\("Whether to draw the nullclines of the \
system"\)\(.\)\)}
                },
              ColumnAlignments->{Left}],
            GridBoxOptions->{RowLines->{True, False}}]],
        "3ColumnBox"], TraditionalForm]], "Text",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic],

Cell[TextData[{
  "Options for ",
  StyleBox["PhasePlot", "MR"],
  "."
}], "Caption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic],

Cell["One more nice example:", "MathCaption"],

Cell[BoxData[
    \(PhasePlot[{\(x \((2 - x - 4  y)\) \((x + 4  y - 3)\)\)\/\(1 + \((x + 4  \
y)\)\^2\), \  .5  y \((2 - 2  x - y)\)}, \ {t, \ \(-10\), \ 20}, \ {x, \ 0, \ 
        2}, \ {y, \ 0, \ 1.5}, 
      InitialPoints \[Rule] {{1.2, 1}, { .8,  .4}}, \ 
      MaxCurves\  \[Rule] {5, 1}, ScaleArrows \[Rule]  .5, 
      ArrowColor \[Rule] GrayLevel[ .6], PlotStyle \[Rule] Hue[0], 
      ShowNullclines \[Rule] True, ShowEquilibriumPoints \[Rule] True, 
      Ticks \[Rule]  .5 {Range[1, 3], Range[1, 4]}]\)], "Input",
  CellLabel->"In[65]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["NDPlot", "SubsectionIcon"],

Cell[TextData[{
  "Here are three examples of the use of ",
  StyleBox["NDPlot",
    FontFamily->"Courier"],
  "."
}], "MathCaption"],

Cell[BoxData[
    RowBox[{"NDPlot", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{
                SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], 
              "==", \(\(-t\)\ y[t]\)}], ",", \(y[0] == 1\)}], "}"}], 
        ",", \(y[t]\), ",", \({t, \(-3\), 3}\)}], "]"}]], "Input",
  CellLabel->"In[66]:="],

Cell[BoxData[
    RowBox[{"NDPlot", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{
                SuperscriptBox["x", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], "==", \(y[t]\)}],
             ",", 
            RowBox[{
              RowBox[{
                SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], 
              "==", \(-Sin[x[t]]\)}], ",", \(x[0] == \(-1\)\), 
            ",", \(y[0] == 1\)}], "}"}], ",", \({x[t], y[t]}\), 
        ",", \({t, 0, 8}\)}], "]"}]], "Input",
  CellLabel->"In[67]:="],

Cell[BoxData[{
    RowBox[{
      RowBox[{"lorenz", "=", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SuperscriptBox["x", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], 
                "==", \(a\ \((y[t] - x[t])\)\)}], ",", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["y", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], 
                "==", \(b\ x[t] - y[t] - x[t]\ z[t]\)}], ",", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["z", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], 
                "==", \(x[t]\ y[t] - c\ z[t]\)}], ",", \(x[0] == \(-8\)\), 
              ",", \(y[0] == 8\), ",", \(z[0] == 27\)}], "}"}], "/.", 
          "\[InvisibleSpace]", \({a \[Rule] 10, b \[Rule] 28, 
            c \[Rule] 8\/3}\)}]}], ";"}], "\n", \(NDPlot[
      lorenz, {x[t], y[t], z[t]}, {t, 0, 30}, MaxSteps \[Rule] \[Infinity], 
      PlotPoints \[Rule] 2000]\)}], "Input",
  CellLabel->"In[68]:="],

Cell[TextData[{
  "Any of the options for ",
  StyleBox["NDSolve",
    FontFamily->"Courier"],
  ", ",
  StyleBox["Plot",
    FontFamily->"Courier"],
  ", ",
  StyleBox["ParametricPlot",
    FontFamily->"Courier"],
  ", or ",
  StyleBox["ParametricPlot3D",
    FontFamily->"Courier"],
  " can be used in ",
  StyleBox["NDPlot",
    FontFamily->"Courier"],
  "."
}], "MathCaption"]
}, Closed]],

Cell[CellGroupData[{

Cell["PoincareTimeSection", "SubsectionIcon"],

Cell[TextData[{
  "Here is an example of the use of ",
  StyleBox["PoincareTimeSection",
    FontFamily->"Courier"],
  "."
}], "MathCaption"],

Cell[BoxData[
    \(PoincareTimeSection[{y, x - x\^3 -  .15\ y +  .3\ Cos[t]}, {t, 0, 5000, 
        2\ \[Pi]}, {x, \(-1\)}, {y, 1}]\)], "Input",
  CellLabel->"In[70]:="],

Cell[TextData[{
  "Options for ",
  StyleBox["PoincareTimeSection",
    FontFamily->"Courier"],
  " are the same as for ",
  StyleBox["ListPlot",
    FontFamily->"Courier"],
  "."
}], "MathCaption"]
}, Closed]],

Cell[CellGroupData[{

Cell["ViewProjections", "SubsectionIcon"],

Cell[TextData[{
  "Here is an example of the use of ",
  StyleBox["ViewProjections",
    FontFamily->"Courier"],
  "."
}], "MathCaption"],

Cell[BoxData[
    \(ViewProjections[{Cos[t], Sin[t]\ Cos[t], Sin[3\ t]}, {t, 0, 
        2\ \[Pi]}, {x, y, z}, PlotPoints \[Rule] 100]\)], "Input",
  CellLabel->"In[71]:="],

Cell[TextData[{
  "Options for ",
  StyleBox["ViewProjections",
    FontFamily->"Courier"],
  " are the same as for ",
  StyleBox["ParametricPlot",
    FontFamily->"Courier"],
  " and ",
  StyleBox["GraphicsArray",
    FontFamily->"Courier"],
  " with the addition of:"
}], "MathCaption"],

Cell[BoxData[
    FormBox[
      StyleBox[
        FrameBox[
          StyleBox[GridBox[{
                {
                  StyleBox[\(option\ name\),
                    FontFamily->"Times",
                    FontSlant->"Italic"], 
                  StyleBox[\(default\ value\),
                    "TI"], "\[Null]"},
                {
                  StyleBox["OuterFrame",
                    FontFamily->"Courier"], 
                  StyleBox["False",
                    "MR",
                    
                    FontFamily->
                      "Courier"], \(Whether\ to\ frame\ the\ entire\ \(\(plot\
\)\(.\)\(\ \)\)\)},
                {
                  StyleBox["OuterFrameStyle",
                    FontFamily->"Courier"], 
                  StyleBox["None",
                    
                    FontFamily->
                      "Courier"], \(\(\ \)\(Style\ specification\ for\ the\ \
outer\ \(\(frame\)\(.\)\)\)\)}
                },
              ColumnAlignments->{Left}],
            GridBoxOptions->{RowLines->{True, False}}]],
        "3ColumnBox"], TraditionalForm]], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["TimeStatePlot", "SubsectionIcon"],

Cell[TextData[{
  "An example of the use of ",
  StyleBox["TimeStatePlot",
    FontFamily->"Courier"],
  ":"
}], "MathCaption"],

Cell[BoxData[
    \(\(TimeStatePlot[\[ExponentialE]\^\(-t\)\ {Sin[3\ \[Pi]\ t], 
            Cos[3\ \[Pi]\ t]}, {t, 0, 4}];\)\)], "Input",
  CellLabel->"In[72]:="],

Cell[TextData[{
  "To plot the solution of a pair of differential equations, one must provide \
",
  StyleBox["TimeStatePlot",
    FontFamily->"Courier"],
  " with the solution."
}], "MathCaption"],

Cell[BoxData[{
    RowBox[{\(tmax = 30\), ";", 
      RowBox[{\(soln[t_]\), ":=", 
        RowBox[{\({x[t], y[t]}\), "/.", "\[InvisibleSpace]", 
          RowBox[{
            RowBox[{"NDSolve", "[", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      RowBox[{
                        SuperscriptBox["x", "\[Prime]",
                          MultilineFunction->None], "[", "t", "]"}], 
                      "==", \(y[t]\)}], ",", 
                    RowBox[{
                      RowBox[{
                        SuperscriptBox["y", "\[Prime]",
                          MultilineFunction->None], "[", "t", "]"}], 
                      "==", \(\(-x[t]\) - \((x[t]\^2 - 1)\)\ y[t]\)}], 
                    ",", \(x[0] == \(-4\)\), ",", \(y[0] == 4\)}], "}"}], 
                ",", \({x[t], y[t]}\), ",", \({t, 0, tmax}\)}], "]"}], 
            "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}]}], 
      ";"}], "\n", \(TimeStatePlot[Evaluate[soln[t]], {t, 0, tmax}, 
        BoxRatios \[Rule] {2, 1, 1}, PlotPoints \[Rule] 150];\)}], "Input",
  CellLabel->"In[73]:="],

Cell[TextData[{
  " ",
  StyleBox["TimeStatePlot",
    FontFamily->"Courier"],
  " uses ",
  StyleBox["ParametricPlot3D",
    FontFamily->"Courier"],
  " and ",
  StyleBox["Shadow",
    FontFamily->"Courier"],
  " from the ",
  StyleBox["Graphics3D",
    FontFamily->"Courier"],
  " package. The options of ",
  StyleBox["ParametricPlot3D",
    FontFamily->"Courier"],
  " can be used, as well as the following:"
}], "MathCaption"],

Cell[BoxData[
    FormBox[
      StyleBox[
        FrameBox[
          StyleBox[GridBox[{
                {
                  StyleBox[\(option\ name\),
                    FontFamily->"Times",
                    FontSlant->"Italic"], 
                  StyleBox[\(default\ value\),
                    "TI"], "\[Null]"},
                {
                  StyleBox["ShadowColors",
                    FontFamily->"Courier"], 
                  
                  StyleBox["\<\"{RGBColor[1,0,0], RGBColor[0,1,0], \
RGBColor[0,0,1]}\"\>",
                    "MR",
                    FontFamily->"Courier"], " "},
                {
                  StyleBox["ShadowPositions",
                    FontFamily->"Courier"], 
                  StyleBox["\<\"{-.67,.8,-.67}\"\>",
                    "MR",
                    FontFamily->"Courier"], " "}
                },
              ColumnAlignments->{Left}],
            GridBoxOptions->{RowLines->{True, False}}]],
        "2ColumnBox"], TraditionalForm]], "Text"],

Cell[TextData[{
  "Options for ",
  StyleBox["TimeStatePlot",
    FontFamily->"Courier"],
  "."
}], "Caption"],

Cell["\<\
Here\[CloseCurlyQuote]s an example in which several options are \
used.\
\>", "MathCaption"],

Cell[BoxData[{
    RowBox[{\(tmax = 30\), ";", 
      RowBox[{\(soln[t_]\), ":=", 
        RowBox[{\({x[t], y[t]}\), "/.", "\[InvisibleSpace]", 
          RowBox[{
            RowBox[{"NDSolve", "[", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      RowBox[{
                        SuperscriptBox["x", "\[Prime]",
                          MultilineFunction->None], "[", "t", "]"}], 
                      "==", \(y[t]\)}], ",", 
                    RowBox[{
                      RowBox[{
                        SuperscriptBox["y", "\[Prime]",
                          MultilineFunction->None], "[", "t", "]"}], 
                      "==", \(-Sin[x[t]]\)}], ",", \(x[0] == 3\), 
                    ",", \(y[0] == 0\)}], "}"}], ",", \({x[t], y[t]}\), 
                ",", \({t, 0, tmax}\)}], "]"}], "\[LeftDoubleBracket]", "1", 
            "\[RightDoubleBracket]"}]}]}], 
      ";"}], "\n", \(TimeStatePlot[Evaluate[soln[t]], {t, 0, tmax}, 
        BoxRatios \[Rule] {2, 1, 1}, PlotPoints \[Rule] 150, 
        PlotStyle \[Rule] {AbsoluteThickness[2], Hue[0]}, 
        ViewPoint \[Rule] {4, \(-3\), 1}, 
        ShadowColors \[Rule] {GrayLevel[ .5], GrayLevel[ .5], 
            GrayLevel[ .5]}];\)}], "Input",
  CellLabel->"In[75]:="],

Cell[TextData[{
  "If you specify ",
  StyleBox["PlotRange",
    FontFamily->"Courier"],
  " (e.g., when making an animation) , you\[CloseCurlyQuote]ll need to make \
adjustments to the ",
  StyleBox["ShadowPositions",
    FontFamily->"Courier"],
  "."
}], "MathCaption"],

Cell[BoxData[
    \(\(TimeStatePlot[\[ExponentialE]\^\(-t\)\ {Sin[3\ \[Pi]\ t], 
            Cos[3\ \[Pi]\ t]}, {t, 0, 1}, PlotPoints \[Rule] 100, 
        ShadowPositions \[Rule] {\(- .5\),  .5, \(- .5\)}, 
        PlotRange \[Rule] {{\(-1\), 4}, {\(-1\), 1}, {\(-1\), 
              1}}];\)\)], "Input",
  CellLabel->"In[77]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["PlotImplicit", "SubsectionIcon"],

Cell[TextData[{
  "Examples of the use of ",
  StyleBox["PlotImplicit",
    FontFamily->"Courier"],
  ":"
}], "MathCaption"],

Cell[BoxData[
    \(PlotImplicit[
      x\^3 - x\ y + y\^3, {x, \(-2\), 2}, {y, \(-3\), 3}]\)], "Input",
  CellLabel->"In[78]:="],

Cell[BoxData[
    \(PlotImplicit[x\^3 - x\ y + y\^3, {x, \(-2\), 2}, {y, \(-3\), 3}, 
      ShowDensity \[Rule] True]\)], "Input",
  CellLabel->"In[79]:="],

Cell[TextData[{
  StyleBox["PlotImplicit",
    FontFamily->"Courier"],
  " uses ",
  StyleBox["ContourPlot",
    FontFamily->"Courier"],
  " and ",
  StyleBox["DensityPlot",
    FontFamily->"Courier"],
  " and accepts any of their options. In addition, the following option can \
be used."
}], "MathCaption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic],

Cell[BoxData[
    StyleBox[
      FrameBox[
        StyleBox[GridBox[{
              {
                StyleBox[\(option\ name\),
                  FontFamily->"Times",
                  FontSlant->"Italic"], 
                StyleBox[\(default\ value\),
                  "TI"], " "},
              {"ShowDensity", "False", 
                
                StyleBox[\(Whether\ to\ include\ a\ density\ plot\ under\ the\
\ \(\(curves\)\(.\)\)\),
                  FontFamily->"Times"]}
              },
            ColumnAlignments->{Left}],
          GridBoxOptions->{RowLines->{True, False}}]],
      "3ColumnBox"]], "Text",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic,
  ShowStringCharacters->False,
  GridBoxOptions->{RowSpacings->1.5,
  GridFrame->False,
  RowLines->False}],

Cell[TextData[{
  "Options for ",
  StyleBox["PlotImplicit",
    FontFamily->"Courier"],
  "."
}], "Caption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->Automatic]
}, Closed]]
},
FrontEndVersion->"4.1 for Macintosh",
ScreenRectangle->{{0, 1600}, {0, 1178}},
WindowSize->{905, 869},
WindowMargins->{{207, Automatic}, {Automatic, 59}},
ShowCellLabel->False,
CellLabelAutoDelete->False,
Magnification->1,
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1705, 50, 107, 5, 35, "Subsection"],
Cell[1815, 57, 122, 3, 21, "Text"],
Cell[1940, 62, 6656, 294, 336, "DefinitionBox",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[8599, 358, 65, 0, 15, "Caption"],
Cell[8667, 360, 46, 0, 27, "MathCaption"],
Cell[8716, 362, 63, 1, 26, "Input"],

Cell[CellGroupData[{
Cell[8804, 367, 32, 0, 35, "SubsectionIcon"],
Cell[8839, 369, 267, 8, 27, "MathCaption"],
Cell[9109, 379, 119, 3, 25, "Input"],
Cell[9231, 384, 163, 3, 25, "Input"],
Cell[9397, 389, 197, 4, 25, "Input"],
Cell[9597, 395, 155, 3, 25, "Input"],
Cell[9755, 400, 297, 10, 27, "MathCaption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[10055, 412, 1408, 39, 128, "Text"],
Cell[11466, 453, 182, 7, 15, "Caption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[11651, 462, 1138, 38, 47, "Text"],
Cell[12792, 502, 1560, 43, 86, "Text",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[14355, 547, 188, 7, 15, "Caption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[14546, 556, 4025, 104, 166, "Text",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[18574, 662, 184, 7, 15, "Caption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[18761, 671, 1841, 47, 83, "Text",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[20605, 720, 186, 7, 15, "Caption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[20794, 729, 1572, 45, 73, "Text",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[22369, 776, 138, 5, 27, "MathCaption"],
Cell[22510, 783, 157, 3, 25, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[22704, 791, 35, 0, 25, "SubsectionIcon"],
Cell[22742, 793, 359, 12, 27, "MathCaption"],
Cell[23104, 807, 227, 4, 26, "Input"],
Cell[23334, 813, 147, 5, 27, "MathCaption"],
Cell[23484, 820, 141, 3, 25, "Input"],
Cell[23628, 825, 100, 3, 27, "MathCaption"],
Cell[23731, 830, 319, 6, 60, "Input"],
Cell[24053, 838, 431, 17, 27, "MathCaption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[24487, 857, 872, 25, 49, "Text",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[25362, 884, 179, 7, 15, "Caption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[25544, 893, 45, 0, 27, "MathCaption"],
Cell[25592, 895, 549, 9, 77, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[26178, 909, 32, 0, 25, "SubsectionIcon"],
Cell[26213, 911, 133, 5, 30, "MathCaption"],
Cell[26349, 918, 413, 11, 26, "Input"],
Cell[26765, 931, 642, 17, 42, "Input"],
Cell[27410, 950, 1139, 26, 109, "Input"],
Cell[28552, 978, 380, 17, 46, "MathCaption"]
}, Closed]],

Cell[CellGroupData[{
Cell[28969, 1000, 45, 0, 25, "SubsectionIcon"],
Cell[29017, 1002, 141, 5, 30, "MathCaption"],
Cell[29161, 1009, 170, 3, 49, "Input"],
Cell[29334, 1014, 198, 8, 30, "MathCaption"]
}, Closed]],

Cell[CellGroupData[{
Cell[29569, 1027, 41, 0, 25, "SubsectionIcon"],
Cell[29613, 1029, 137, 5, 30, "MathCaption"],
Cell[29753, 1036, 172, 3, 42, "Input"],
Cell[29928, 1041, 288, 11, 46, "MathCaption"],
Cell[30219, 1054, 1115, 31, 71, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[31371, 1090, 39, 0, 25, "SubsectionIcon"],
Cell[31413, 1092, 127, 5, 30, "MathCaption"],
Cell[31543, 1099, 163, 3, 27, "Input"],
Cell[31709, 1104, 197, 6, 30, "MathCaption"],
Cell[31909, 1112, 1160, 24, 98, "Input"],
Cell[33072, 1138, 431, 17, 46, "MathCaption"],
Cell[33506, 1157, 1019, 28, 71, "Text"],
Cell[34528, 1187, 110, 5, 15, "Caption"],
Cell[34641, 1194, 102, 3, 30, "MathCaption"],
Cell[34746, 1199, 1325, 28, 122, "Input"],
Cell[36074, 1229, 271, 9, 46, "MathCaption"],
Cell[36348, 1240, 330, 6, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[36715, 1251, 38, 0, 25, "SubsectionIcon"],
Cell[36756, 1253, 124, 5, 30, "MathCaption"],
Cell[36883, 1260, 129, 3, 30, "Input"],
Cell[37015, 1265, 155, 3, 30, "Input"],
Cell[37173, 1270, 402, 14, 46, "MathCaption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[37578, 1286, 829, 25, 77, "Text",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic],
Cell[38410, 1313, 203, 8, 15, "Caption",
  PageBreakWithin->Automatic,
  PageBreakBelow->Automatic]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

