(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     17746,        594]*)
(*NotebookOutlinePosition[     18465,        619]*)
(*  CellTagsIndexPosition[     18421,        615]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  "DiffEqs",
  StyleBox["`", "MB"],
  "DETools",
  StyleBox["`", "MB"]
}], "Subsection"],

Cell["\<\
This package provides the following functions for solving \
differential equations. \
\>", "Text"],

Cell[TextData[{
  StyleBox["FOLDSolve[",
    FontFamily->"Courier"],
  Cell[BoxData[
      FormBox[
        StyleBox["eqn",
          FontSlant->"Italic"], TraditionalForm]]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`y[t], \ t\)]],
  StyleBox["]",
    FontFamily->"Courier"],
  "\n\tsolves a first-order linear differential equation for ",
  Cell[BoxData[
      \(TraditionalForm\`y(t)\)]],
  ".\n\t\n",
  StyleBox["BernoulliDSolve[",
    FontFamily->"Courier"],
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox["eqn",
            FontSlant->"Italic"], ",", " ", \(y[t]\), ",", " ", "t"}], 
        TraditionalForm]]],
  StyleBox["]",
    FontFamily->"Courier"],
  "\n\tsolves a Bernoulli-type differential equation for ",
  Cell[BoxData[
      \(TraditionalForm\`y(t)\)]],
  ".\n\t\n",
  StyleBox["RiccatiDSolve[",
    FontFamily->"Courier"],
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox["eqn",
            FontSlant->"Italic"], ",", " ", \(y[t]\), ",", " ", "t", ",", 
          " ", \(z[t]\)}], TraditionalForm]]],
  StyleBox["]",
    FontFamily->"Courier"],
  "\n\tsolves a Riccati-type differential equation for ",
  Cell[BoxData[
      \(TraditionalForm\`y(t)\)]],
  ", given a known solution ",
  Cell[BoxData[
      \(TraditionalForm\`z[t]\)]],
  ".\n\t\n",
  StyleBox["SeparableDSolve[",
    FontFamily->"Courier"],
  Cell[BoxData[
      \(TraditionalForm\`f[x]\ \[DifferentialD]x\  == \ 
        g[y]\ \[DifferentialD]y\)]],
  StyleBox["]",
    FontFamily->"Courier"],
  "\n\treturns an implicit solution of a separable differential equation.\n\n\
",
  StyleBox["SeriesDSolve[",
    FontFamily->"Courier"],
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox["eqn",
            FontSlant->"Italic"], ",", " ", \(y[t]\), ",", 
          " ", \({t, \ t\_0, \ n}\)}], TraditionalForm]]],
  StyleBox["]",
    FontFamily->"Courier"],
  "\n\treturns the first n terms of a series expansion about t0 of the \
solution\n\tof a first- or second-order differential equation.\n"
}], "Text",
  CellFrame->True,
  CellMargins->{{Inherited, Inherited}, {4, Inherited}},
  ParagraphSpacing->{0, 0},
  Background->RGBColor[1, 0.6, 0.6]],

Cell["Finding solutions of differential equations. ", "Caption"],

Cell["This loads the package.", "MathCaption"],

Cell[BoxData[
    \(<< "\<DiffEqs`DETools`\>"\)], "Input",
  CellLabel->"In[6]:="],

Cell[CellGroupData[{

Cell["FOLDSolve", "SubsectionIcon"],

Cell[TextData[{
  "A few examples of the use of ",
  StyleBox["FOLDSolve",
    FontFamily->"Courier"],
  ":"
}], "Theorem"],

Cell[BoxData[
    RowBox[{"FOLDSolve", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{
              SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], 
            "+", \(p[t]\ y[t]\)}], "==", \(f[t]\)}], ",", \(y[t]\), ",", 
        "t"}], "]"}]], "Input",
  CellLabel->"In[7]:="],

Cell[BoxData[
    RowBox[{"FOLDSolve", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{
                  SuperscriptBox["y", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], 
                "+", \(p[t]\ y[t]\)}], "==", \(f[t]\)}], 
            ",", \(y[0] == y0\)}], "}"}], ",", \(y[t]\), ",", "t"}], 
      "]"}]], "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    RowBox[{"FOLDSolve", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{
              SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "+", \(t\ y[t]\)}],
           "==", \(Cos[t]\)}], ",", \(y[t]\), ",", "t", 
        ",", \(ConstantName \[Rule] \[Mu]\)}], "]"}]], "Input",
  CellLabel->"In[9]:="],

Cell[BoxData[
    RowBox[{
      RowBox[{"FOLDSolve", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{
                RowBox[{
                  RowBox[{
                    SuperscriptBox["y", "\[Prime]",
                      MultilineFunction->None], "[", "t", "]"}], 
                  "-", \(y[t]\/\(2 - t\)\)}], "==", "t"}], 
              ",", \(y[0] == 1\)}], "}"}], ",", \(y[t]\), ",", "t"}], "]"}], "//",
       "Simplify"}]], "Input",
  CellLabel->"In[10]:="],

Cell[TextData[{
  "Here\[CloseCurlyQuote]s an example on which ",
  StyleBox["DSolve",
    FontFamily->"Courier"],
  " works for a ",
  StyleBox["very",
    FontSlant->"Italic"],
  " long time."
}], "MathCaption"],

Cell[BoxData[
    RowBox[{"FOLDSolve", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{
                  SuperscriptBox["y", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], 
                "-", \(Sin[t]\ y[t]\)}], "==", \(Log[t]\^2\)}], 
            ",", \(y[0] == 1\)}], "}"}], ",", \(y[t]\), ",", "t"}], 
      "]"}]], "Input",
  CellLabel->"In[11]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["BernoulliDSolve", "SubsectionIcon"],

Cell[TextData[{
  "A few examples of the use of ",
  StyleBox["BernoulliDSolve",
    FontFamily->"Courier"],
  ":"
}], "Theorem"],

Cell[BoxData[
    RowBox[{"BernoulliDSolve", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{
              SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "+", \(y[t]\)}], 
          "==", \(-y[t]\^2\)}], ",", \(y[t]\), ",", "t"}], "]"}]], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    RowBox[{"BernoulliDSolve", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{
              SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "+", \(t\ y[t]\)}],
           "==", \(\@y[t]\)}], ",", \(y[t]\), ",", "t", 
        ",", \(ConstantName \[Rule] \[Gamma]\)}], "]"}]], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(BernoulliDSolve[{\(y\^\[Prime]\)[t] - 
              t\^3\ y[t] \[Equal] \(-t\)\ y[t]\^3, y[0] \[Equal] 1}, y[t], 
        t] // Simplify\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    RowBox[{"BernoulliDSolve", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{
                  SuperscriptBox["y", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], 
                "-", \(Cos[t]\ y[t]\)}], 
              "==", \(\(-Sin[t]\)\ y[t]\^\(1/3\)\)}], ",", \(y[0] == 1\)}], 
          "}"}], ",", \(y[t]\), ",", "t"}], "]"}]], "Input",
  CellLabel->"In[15]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["RiccatiDSolve", "SubsectionIcon"],

Cell[TextData[{
  "A few examples of the use of ",
  StyleBox["RiccatiDSolve",
    FontFamily->"Courier"],
  ":"
}], "Theorem"],

Cell["The fourth argument is a known solution.", "MathCaption"],

Cell[BoxData[
    RowBox[{"RiccatiDSolve", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{
              SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "+", \(y[t]\)}], 
          "==", \(\(-y[t]\^2\) + \[ExponentialE]\^\(\(-2\)\ t\)\)}], 
        ",", \(y[t]\), ",", "t", ",", \(\[ExponentialE]\^\(-t\)\)}], 
      "]"}]], "Input",
  CellLabel->"In[16]:="],

Cell[BoxData[
    RowBox[{
      RowBox[{"RiccatiDSolve", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{
                RowBox[{
                  RowBox[{
                    SuperscriptBox["y", "\[Prime]",
                      MultilineFunction->None], "[", "t", "]"}], 
                  "-", \(y[t]\/\(t\ Log[t]\)\)}], 
                "==", \(Log[t]\^2 - y[t]\^2\)}], 
              ",", \(y[\[ExponentialE]] == 0\)}], "}"}], ",", \(y[t]\), ",", 
          "t", ",", \(Log[t]\)}], "]"}], "//", "Simplify"}]], "Input",
  CellLabel->"In[17]:="],

Cell[TextData[{
  "Here\[CloseCurlyQuote]s an example on which ",
  StyleBox["DSolve",
    FontFamily->"Courier"],
  " fails."
}], "MathCaption"],

Cell[BoxData[
    RowBox[{
      RowBox[{"RiccatiDSolve", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SuperscriptBox["y", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], 
                "==", \(1 - Sin[t] - Sin[t]\^2 - y[t]\^2\)}], 
              ",", \(y[0] == 0\)}], "}"}], ",", \(y[t]\), ",", "t", 
          ",", \(Cos[t]\)}], "]"}], "//", "Simplify"}]], "Input",
  CellLabel->"In[18]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["SeparableDSolve", "SubsectionIcon"],

Cell[TextData[{
  "A few examples of the use of ",
  StyleBox["SeparableDSolve",
    FontFamily->"Courier"],
  ":"
}], "Theorem"],

Cell["The equation is given in separated form.", "MathCaption"],

Cell[BoxData[
    \(SeparableDSolve[\(x\^2\) \[DifferentialD]x \[Equal] 
        Cos[y] \[DifferentialD]y]\)], "Input",
  CellLabel->"In[19]:="],

Cell[TextData[{
  "The next example uses the ",
  StyleBox["InitialPoint",
    FontFamily->"Courier"],
  " option. Note that the first number is a value for the variable on the \
left side of the equation and the second number is a value for the variable \
on the right side of the equation. Here the initial condition is that y=0 \
when t=1."
}], "MathCaption"],

Cell[BoxData[
    \(SeparableDSolve[
      y\ \[DifferentialD]y \[Equal] 
        t\ \(\[ExponentialE]\^\(-t\)\) \[DifferentialD]t, 
      InitialPoint \[Rule] {0, 1}]\)], "Input",
  CellLabel->"In[20]:="],

Cell[TextData[{
  "Since ",
  StyleBox["SeparableDSolve",
    FontFamily->"Courier"],
  " returns an equation, it can be combined easily with ",
  StyleBox["Solve",
    FontFamily->"Courier"],
  "."
}], "MathCaption"],

Cell[BoxData[
    \(Solve[
      SeparableDSolve[\[DifferentialD]y\/\(1 + y\^2\) == 
          t\ \[DifferentialD]t], y]\)], "Input",
  CellLabel->"In[21]:="],

Cell[TextData[{
  "A result from ",
  StyleBox["SeparableDSolve",
    FontFamily->"Courier"],
  " is easy to feed into ",
  StyleBox["PlotImplicit",
    FontFamily->"Courier"],
  " from the ",
  StyleBox["DEGraphics",
    FontFamily->"Courier"],
  " package."
}], "MathCaption"],

Cell[BoxData[
    \(<< DiffEqs`DEGraphics`\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \(PlotImplicit[
      First[SeparableDSolve[
          Cos[y]\ \[DifferentialD]y == \((1 - 
                  t\^2)\)\ \[DifferentialD]t]], {t, \(-2\), 3}, {y, \(-2\), 
        5}, ShowDensity \[Rule] True]\)], "Input",
  CellLabel->"In[23]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["SeriesDSolve", "SubsectionIcon"],

Cell[TextData[{
  "A few examples of the use of ",
  StyleBox["SeriesDSolve",
    FontFamily->"Courier"],
  ":"
}], "Theorem"],

Cell["A nonlinear first-order initial-value problem:", "MathCaption"],

Cell[BoxData[
    RowBox[{"SeriesDSolve", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{
                SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], 
              "==", \(t - Cos[y[t]]\)}], ",", \(y[0] == \[Pi]\)}], "}"}], 
        ",", \(y[t]\), ",", \({t, 0, 11}\)}], "]"}]], "Input",
  CellLabel->"In[24]:="],

Cell["A nonlinear second-order initial-value problem:", "MathCaption"],

Cell[BoxData[
    RowBox[{"SeriesDSolve", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{
                SuperscriptBox["y", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}], 
              "==", \(t - Cos[y[t]]\)}], ",", \(y[0] == 0\), ",", 
            RowBox[{
              RowBox[{
                SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "0", "]"}], "==", "1"}]}], 
          "}"}], ",", \(y[t]\), ",", \({t, 0, 11}\)}], "]"}]], "Input",
  CellLabel->"In[25]:="],

Cell[TextData[{
  "For a linear second-order equation (with no initial conditions), ",
  StyleBox["SeriesDSolve",
    FontFamily->"Courier"],
  " returns partial sums of a pair of \[OpenCurlyDoubleQuote]fundamental\
\[CloseCurlyDoubleQuote] solutions, if possible."
}], "MathCaption"],

Cell[BoxData[
    RowBox[{"SeriesDSolve", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{
              SuperscriptBox["y", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], 
            "+", \(t\^2\ y[t]\)}], "==", "0"}], ",", \(y[t]\), 
        ",", \({t, 0, 20}\)}], "]"}]], "Input",
  CellLabel->"In[26]:="],

Cell["Here we get expansions about a point other than 0.", "MathCaption"],

Cell[BoxData[
    RowBox[{"SeriesDSolve", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{
              SuperscriptBox["y", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "+", \(y[t]\)}], "==",
           "0"}], ",", \(y[t]\), ",", \({t, \(-1\), 10}\)}], "]"}]], "Input",
  CellLabel->"In[27]:="],

Cell["Nonpolynomial coefficients are no problem.", "MathCaption"],

Cell[BoxData[
    RowBox[{"SeriesDSolve", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{
              SuperscriptBox["y", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "+", 
            RowBox[{\(Sin[t]\), " ", 
              RowBox[{
                SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}]}], 
            "+", \(\[ExponentialE]\^\(-t\)\ y[t]\)}], "==", "0"}], 
        ",", \(y[t]\), ",", \({t, 0, 6}\)}], "]"}]], "Input",
  CellLabel->"In[28]:="],

Cell[TextData[{
  "If there is a nonhomogeneous term,  ",
  StyleBox["SeriesDSolve",
    FontFamily->"Courier"],
  " returns the \[OpenCurlyDoubleQuote]rest\[CloseCurlyDoubleQuote] solution \
(i.e., the solution satisfying zero initial conditions)."
}], "MathCaption"],

Cell[BoxData[
    RowBox[{"SeriesDSolve", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{
              SuperscriptBox["y", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "+", 
            RowBox[{"t", " ", 
              RowBox[{
                SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}]}], 
            "+", \(y[t]\)}], "==", \(Cos[t]\)}], ",", \(y[t]\), 
        ",", \({t, 0, 15}\)}], "]"}]], "Input",
  CellLabel->"In[29]:="],

Cell["\<\
Complex powers and Frobenius-series solutions are found as \
well.\
\>", "MathCaption"],

Cell[BoxData[{
    RowBox[{"SeriesDSolve", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{\(t\^2\), " ", 
              RowBox[{
                SuperscriptBox["y", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}]}], "+", 
            RowBox[{"t", " ", 
              RowBox[{
                SuperscriptBox["y", "\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}]}], 
            "+", \(2\ y[t]\)}], "==", "0"}], ",", \(y[t]\), 
        ",", \({t, 0, 5}\)}], "]"}], "\n", \(Simplify[ComplexExpand[%], 
      t > 0]\)}], "Input",
  CellLabel->"In[30]:="],

Cell[BoxData[
    RowBox[{"SeriesDSolve", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"3", " ", "t", " ", 
              RowBox[{
                SuperscriptBox["y", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "t", "]"}]}], "+", 
            RowBox[{
              SuperscriptBox["y", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], "+", \(t\ y[t]\)}],
           "==", "0"}], ",", \(y[t]\), ",", \({t, 0, 8}\)}], "]"}]], "Input",
  CellLabel->"In[32]:="]
}, Closed]]
},
FrontEndVersion->"4.1 for Macintosh",
ScreenRectangle->{{0, 1600}, {0, 1178}},
WindowSize->{697, 661},
WindowMargins->{{308, Automatic}, {Automatic, 17}},
ShowCellLabel->False,
ShowCellTags->True,
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1705, 50, 104, 5, 35, "Subsection"],
Cell[1812, 57, 108, 3, 21, "Text"],
Cell[1923, 62, 2193, 72, 236, "Text"],
Cell[4119, 136, 64, 0, 15, "Caption"],
Cell[4186, 138, 46, 0, 27, "MathCaption"],
Cell[4235, 140, 82, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[4342, 146, 35, 0, 35, "SubsectionIcon"],
Cell[4380, 148, 123, 5, 30, "Theorem"],
Cell[4506, 155, 352, 10, 26, "Input"],
Cell[4861, 167, 458, 13, 26, "Input"],
Cell[5322, 182, 386, 10, 26, "Input"],
Cell[5711, 194, 513, 14, 41, "Input"],
Cell[6227, 210, 213, 8, 30, "MathCaption"],
Cell[6443, 220, 465, 13, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[6945, 238, 41, 0, 25, "SubsectionIcon"],
Cell[6989, 240, 129, 5, 30, "Theorem"],
Cell[7121, 247, 346, 9, 30, "Input"],
Cell[7470, 258, 396, 10, 32, "Input"],
Cell[7869, 270, 195, 4, 30, "Input"],
Cell[8067, 276, 494, 13, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[8598, 294, 39, 0, 25, "SubsectionIcon"],
Cell[8640, 296, 127, 5, 30, "Theorem"],
Cell[8770, 303, 63, 0, 30, "MathCaption"],
Cell[8836, 305, 431, 11, 30, "Input"],
Cell[9270, 318, 593, 15, 61, "Input"],
Cell[9866, 335, 145, 5, 30, "MathCaption"],
Cell[10014, 342, 510, 13, 47, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[10561, 360, 41, 0, 25, "SubsectionIcon"],
Cell[10605, 362, 129, 5, 30, "Theorem"],
Cell[10737, 369, 63, 0, 30, "MathCaption"],
Cell[10803, 371, 144, 3, 30, "Input"],
Cell[10950, 376, 362, 8, 62, "MathCaption"],
Cell[11315, 386, 205, 5, 27, "Input"],
Cell[11523, 393, 217, 8, 30, "MathCaption"],
Cell[11743, 403, 158, 4, 43, "Input"],
Cell[11904, 409, 278, 11, 30, "MathCaption"],
Cell[12185, 422, 80, 2, 26, "Input"],
Cell[12268, 426, 262, 6, 49, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[12567, 437, 38, 0, 25, "SubsectionIcon"],
Cell[12608, 439, 126, 5, 30, "Theorem"],
Cell[12737, 446, 69, 0, 30, "MathCaption"],
Cell[12809, 448, 420, 11, 26, "Input"],
Cell[13232, 461, 70, 0, 30, "MathCaption"],
Cell[13305, 463, 598, 15, 26, "Input"],
Cell[13906, 480, 284, 6, 46, "MathCaption"],
Cell[14193, 488, 370, 10, 30, "Input"],
Cell[14566, 500, 73, 0, 30, "MathCaption"],
Cell[14642, 502, 358, 9, 26, "Input"],
Cell[15003, 513, 65, 0, 30, "MathCaption"],
Cell[15071, 515, 566, 14, 27, "Input"],
Cell[15640, 531, 268, 6, 46, "MathCaption"],
Cell[15911, 539, 542, 14, 26, "Input"],
Cell[16456, 555, 97, 3, 30, "MathCaption"],
Cell[16556, 560, 635, 16, 47, "Input"],
Cell[17194, 578, 536, 13, 26, "Input"]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

