(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      6585,        238]*)
(*NotebookOutlinePosition[      7304,        263]*)
(*  CellTagsIndexPosition[      7260,        259]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  "DiffEqs",
  StyleBox["`", "MB"],
  "FourierTools",
  StyleBox["`", "MB"]
}], "Subsection"],

Cell["\<\
This package provides functions for computing partial sums of \
Fourier series. \
\>", "Text"],

Cell[TextData[{
  StyleBox["FourierSum[",
    FontFamily->"Courier"],
  Cell[BoxData[
      \(TraditionalForm\`f[x]\)]],
  ", {",
  Cell[BoxData[
      \(TraditionalForm\`x, \(-L\), L\)]],
  "}, ",
  StyleBox["m",
    FontSlant->"Italic"],
  StyleBox["]",
    FontFamily->"Courier"],
  " \n\treturns the first ",
  StyleBox["m",
    FontSlant->"Italic"],
  " terms of the Fourier series of ",
  Cell[BoxData[
      \(TraditionalForm\`f[x]\)]],
  " on ",
  Cell[BoxData[
      \(TraditionalForm\`\(\([\)\(\(-L\), L\)\(]\)\)\)]],
  ".\n\t\n",
  StyleBox["NFourierSum[",
    FontFamily->"Courier"],
  Cell[BoxData[
      \(TraditionalForm\`f[x]\)]],
  ", {",
  Cell[BoxData[
      \(TraditionalForm\`x, \(-L\), L\)]],
  "}, ",
  StyleBox["m",
    FontSlant->"Italic"],
  StyleBox["]",
    FontFamily->"Courier"],
  " \n\treturns the first m terms of the Fourier series of ",
  Cell[BoxData[
      \(TraditionalForm\`f[x]\)]],
  " on ",
  Cell[BoxData[
      \(TraditionalForm\`\(\([\)\(\(-L\), L\)\(]\)\)\)]],
  " with numerical coefficients.\n\t\n",
  StyleBox["FourierCosSum[",
    FontFamily->"Courier"],
  Cell[BoxData[
      \(TraditionalForm\`f[x]\)]],
  ", {",
  Cell[BoxData[
      \(TraditionalForm\`x, 0, L\)]],
  "}, ",
  StyleBox["m",
    FontSlant->"Italic"],
  StyleBox["]\n\t",
    FontFamily->"Courier"],
  " returns the first ",
  StyleBox["m",
    FontSlant->"Italic"],
  " terms of the Fourier cosine series of ",
  Cell[BoxData[
      \(TraditionalForm\`f[x]\)]],
  " on ",
  Cell[BoxData[
      \(TraditionalForm\`\(\([\)\(0, L\)\(]\)\)\)]],
  ".\n\n",
  StyleBox["NFourierCosSum[",
    FontFamily->"Courier"],
  Cell[BoxData[
      \(TraditionalForm\`f[x]\)]],
  ", {",
  Cell[BoxData[
      \(TraditionalForm\`x, 0, L\)]],
  "}, ",
  StyleBox["m",
    FontSlant->"Italic"],
  StyleBox["]",
    FontFamily->"Courier"],
  " \n\treturns the first ",
  StyleBox["m",
    FontSlant->"Italic"],
  " terms of the Fourier cosine series of ",
  Cell[BoxData[
      \(TraditionalForm\`f[x]\)]],
  " on ",
  Cell[BoxData[
      \(TraditionalForm\`\(\([\)\(0, L\)\(]\)\)\)]],
  " with numerical coefficients.\n\n",
  StyleBox["FourierSinSum[",
    FontFamily->"Courier"],
  Cell[BoxData[
      \(TraditionalForm\`f[x]\)]],
  ", {",
  Cell[BoxData[
      \(TraditionalForm\`x, 0, L\)]],
  "}, ",
  StyleBox["m",
    FontSlant->"Italic"],
  StyleBox["]",
    FontFamily->"Courier"],
  " \n\treturns the first m terms of the Fourier sine series of ",
  Cell[BoxData[
      \(TraditionalForm\`f[x]\)]],
  " on ",
  Cell[BoxData[
      \(TraditionalForm\`\(\([\)\(0, L\)\(]\)\)\)]],
  ".\n\n",
  StyleBox["NFourierSinSum[",
    FontFamily->"Courier"],
  Cell[BoxData[
      \(TraditionalForm\`f[x]\)]],
  ", {",
  Cell[BoxData[
      \(TraditionalForm\`x, 0, L\)]],
  "}, ",
  StyleBox["m",
    FontSlant->"Italic"],
  StyleBox["]\n\t",
    FontFamily->"Courier"],
  " returns the first m terms of the Fourier sine series of ",
  Cell[BoxData[
      \(TraditionalForm\`f[x]\)]],
  " on ",
  Cell[BoxData[
      \(TraditionalForm\`\(\([\)\(0, L\)\(]\)\)\)]],
  " with numerical coefficients.\n"
}], "Text",
  CellFrame->True,
  CellMargins->{{Inherited, Inherited}, {4, Inherited}},
  ParagraphSpacing->{0, 0},
  Background->RGBColor[1, 0.6, 0.6]],

Cell["Computing terms of Fourier series. ", "Caption"],

Cell["This loads the package.", "MathCaption"],

Cell["<< DiffEqs`FourierTools`", "Input",
  CellLabel->"In[157]:="],

Cell[TextData[{
  "The following are a few examples that illustrate the use of the functions \
in ",
  StyleBox["FourierTools",
    FontFamily->"Courier"],
  "."
}], "MathCaption"],

Cell[BoxData[{
    \(FourierSum[UnitStep[x - 1\/2], {x, \(-1\), 1}, 10]\), "\n", 
    \(Plot[{UnitStep[x - 1\/2], %}, {x, \(-1\), 1}]\)}], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[{
    \(NFourierSum[Sin[Cos[x\^2 - x]], {x, \(-\[Pi]\), \[Pi]}, 
      6]\), "\[IndentingNewLine]", 
    \(Plot[{Sin[Cos[x\^2 - x]], %}, {x, \(-\[Pi]\), \[Pi]}]\)}], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[{
    \(FourierCosSum[x, {x, 0, 1}, 3]\), "\[IndentingNewLine]", 
    \(Plot[{x, %}, {x, \(-1\), 1}]\)}], "Input",
  CellLabel->"In[218]:="],

Cell[BoxData[{
    \(FourierSinSum[x, {x, 0, 1}, 5]\), "\[IndentingNewLine]", 
    \(Plot[{x, %}, {x, \(-1\), 1}]\)}], "Input",
  CellLabel->"In[220]:="],

Cell[BoxData[{
    \(NFourierCosSum[ArcTan[x], {x, 0, 1}, 5]\), "\[IndentingNewLine]", 
    \(Plot[{ArcTan[x], %}, {x, \(-1\), 1}]\)}], "Input",
  CellLabel->"In[224]:="],

Cell[BoxData[{
    \(NFourierSinSum[ArcTan[x], {x, 0, 1}, 10]\), "\[IndentingNewLine]", 
    \(Plot[{ArcTan[x], %}, {x, \(-1\), 1}]\)}], "Input",
  CellLabel->"In[226]:="]
},
FrontEndVersion->"4.1 for Macintosh",
ScreenRectangle->{{0, 1600}, {0, 1178}},
WindowSize->{742, 753},
WindowMargins->{{241, Automatic}, {Automatic, 75}},
ShowCellLabel->False,
ShowCellTags->True,
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1705, 50, 109, 5, 35, "Subsection"],
Cell[1817, 57, 104, 3, 21, "Text"],
Cell[1924, 62, 3249, 128, 262, "Text"],
Cell[5176, 192, 54, 0, 15, "Caption"],
Cell[5233, 194, 46, 0, 27, "MathCaption"],
Cell[5282, 196, 67, 1, 26, "Input"],
Cell[5352, 199, 180, 6, 27, "MathCaption"],
Cell[5535, 207, 172, 3, 73, "Input"],
Cell[5710, 212, 212, 4, 43, "Input"],
Cell[5925, 218, 153, 3, 40, "Input"],
Cell[6081, 223, 153, 3, 40, "Input"],
Cell[6237, 228, 170, 3, 40, "Input"],
Cell[6410, 233, 171, 3, 40, "Input"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

